@@ -25,6 +25,7 @@
  *      and vice versa.
  *		 
  *   -i/--input [pPnNsStTa] --- Input Source:
+ *   -i/--input input[:norm]
  *      'p': PAL       Composite Input or first Bt8x8 input
  *      'P': PAL       SVHS-Input or second Bt8x8 input
  *      't': PAL       TV tuner input or third Bt8x8 input
@@ -34,7 +35,9 @@
  *      's': SECAM     Composite Input or first Bt8x8 input
  *      'S': SECAM     SVHS-Input or second Bt8x8 input
  *      'f': SECAM     TV tuner input or third Bt8x8 input
- *      'a': (or every other letter) Autosense (default)
+ *      'a': Autosense (default)
+ *      input - a number, 1-10
+ *      norm  - pal, ntsc or secam
  *
  *   -d/--decimation num --- Frame recording decimation:
  *      must be either 1, 2 or 4 for identical decimation
@@ -228,7 +231,7 @@ static void Usage(char *progname)
            ""
 #endif
            "]         Format AVI/Quicktime\n");
-	fprintf(stderr, "  -i/--input [pPnNsStTfa]     Input Source\n");
+	fprintf(stderr, "  -i/--input [pPnNsStTfa]     Input Source or 1-10:norm see manpage\n");
 	fprintf(stderr, "  -d/--decimation num         Decimation (either 1,2,4 or two digit number)\n");
 	fprintf(stderr, "  -g/--geometry WxH+X+Y       X-style geometry string (part of 768/720x576/480)\n");
 	fprintf(stderr, "  -q/--quality num            Quality [%%]\n");
@@ -573,6 +576,8 @@ static int set_option(const char *name, char *value)
 {
 	/* return 1 means error, return 0 means okay */
 	int nerr = 0;
+	int i = 0;
+	char *norm_i = NULL;
 
 	if (strcmp(name, "format")==0 || strcmp(name, "f")==0)
 	{
@@ -604,7 +609,42 @@ static int set_option(const char *name, char *value)
 			case 't': info->video_norm = 0 /* PAL */; info->video_src = 2 /* TV-tuner */; break;
 			case 'T': info->video_norm = 1 /* NTSC */; info->video_src = 2 /* TV-tuner */; break;
 			case 'f': info->video_norm = 2 /* SECAM */; info->video_src = 2 /* TV-tuner */; break;
-			default:  info->video_norm = 3 /* auto */; info->video_src = 3 /* auto */; break;
+			case 'a':  info->video_norm = 3 /* auto */; info->video_src = -1 /* auto */; break;
+			default:
+				i = atoi(value);
+				if (i>0 && i<11)
+					{
+					info->video_norm = 3 /* auto */; info->video_src = i - 1 /* input */;
+					norm_i = strstr(value, ":");
+					if(norm_i != NULL && strlen(norm_i) > 1)
+						{
+							if (strcasecmp(norm_i + 1, "pal") == 0)
+								{
+									info->video_norm = 0;
+								}
+							else if (strcasecmp(norm_i + 1, "ntsc") == 0)
+								{
+									info->video_norm = 1;
+								}
+							else if (strcasecmp(norm_i + 1, "secam") == 0)
+								{
+									info->video_norm = 0;
+								}
+							else
+								{
+									mjpeg_error("unknown norm: '%s'", norm_i + 1);
+									nerr++;
+									break;
+								}
+						}
+					} 
+				else 
+					{
+						mjpeg_error("numeric input must be between 1 and 10");
+						nerr++;
+					}
+			break;
+
 		}
 		input_source = value[0];
 	}
@@ -960,10 +1000,10 @@ static void check_command_line_options(int argc, char *argv[])
         if(info->video_format == '\0') {
             if((dotptr = strrchr(argv[optind], '.'))) {
 #ifdef HAVE_LIBQUICKTIME
-                if(!strcasecmp(dotptr+1, "mov") || !strcasecmp(dotptr+1, "qt")
-                    || !strcasecmp(dotptr+1, "moov")) info->video_format = 'q';
+                if (!strcasecmp(dotptr+1, "mov"))
+                    info->video_format = 'q';
 #endif
-                if(!strcasecmp(dotptr+1, "avi")) info->video_format = 'a';
+                if (!strcasecmp(dotptr+1, "avi")) info->video_format = 'a';
             }
             if(info->video_format == '\0') info->video_format = 'a';
         }
@@ -995,6 +1035,7 @@ static void lavrec_print_properties(void)
 		case 't': source = info->software_encoding?"BT8x8 3rd input PAL\n":"PAL TV-tuner\n"; break;
 		case 'T': source = info->software_encoding?"BT8x8 3rd input PAL\n":"NTSC TV-tuner\n"; break;
 		case 'f': source = info->software_encoding?"BT8x8 3rd input PAL\n":"SECAM TV-tuner\n"; break;
+//sam Fix numeric source
 		default:  source = "Auto detect\n";
 	}
 	mjpeg_info("Input Source:       %s", source);
@@ -1053,9 +1094,9 @@ static void print_summary(void)
     ((info->video_norm==1)?(30000/1001):25.));
 
   printf("Recording time  : %2d.%2.2d.%2.2d:%2.2d\n"
-         "Lost frames     : %3.3d\n"
-         "A/V sync ins/del: %3.3d/%3.3d\n"
-         "Audio errors    : %3.3d\n",
+         "Lost frames     : %d\n"
+         "A/V sync ins/del: %d/%d\n"
+         "Audio errors    : %d\n",
     tc.h, tc.m, tc.s, tc.f,
     num_lost, num_ins, num_del, num_aerr);
 }
@@ -1139,7 +1180,12 @@ int main(int argc, char **argv)
   /* Catch and forward selected signals to the signal catcher */
   forward_signals(&sigmask);
 
-  lavrec_main(info);
+  if ((lavrec_main(info)) != 1)
+		{
+			mjpeg_error_exit1("Something went wrong while setting up the card");
+	  		print_summary();
+			lavrec_free(info);
+		}
 
   if (!wait_for_start)
      lavrec_start(info);
