@@ -402,7 +402,10 @@ int decode_png(const char *pngname, int process, parameters_t *param)
     }
   png_read_end(png_ptr, info_ptr);
 #endif  
-  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
+  if (setjmp(png_ptr->jmpbuf)) {
+    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
+    return 2;
+    }
 
   fclose(pngfile);
 
@@ -425,7 +428,7 @@ static int init_parse_files(parameters_t *param)
   if (decode_png(pngname, 0, param) == -1)
     mjpeg_error_exit1("Reading of %s failed.\n", pngname);
 
-  mjpeg_info("Image dimensions are %ldx%ld",
+  mjpeg_info("Image dimensions are %ux%u",
 	     param->width, param->height);
   
   mjpeg_info("Movie frame rate is:  %f frames/second",
@@ -455,7 +458,7 @@ static int init_parse_files(parameters_t *param)
       param->height *= 2;
       mjpeg_info("Non-interleaved fields (image height doubled)");
     }
-  mjpeg_info("Frame size:  %ld x %ld", param->width, param->height);
+  mjpeg_info("Frame size:  %u x %u", param->width, param->height);
 
   return 0;
 }
@@ -553,7 +556,7 @@ static int generate_YUV4MPEG(parameters_t *param)
 		  mjpeg_info("Processing interlaced, top-first %s", pngname);
 #if 0 
 		  decode_jpeg_raw(jpegdata, jpegsize,
-				  Y4M_ILACE_TOP_FIRST, /* gz: was: LAV_INTER_TOP_FIRST */
+				  Y4M_ILACE_TOP_FIRST,
 				  420, param->width, param->height,
 				  yuv[0], yuv[1], yuv[2]);
 #endif
@@ -562,7 +565,7 @@ static int generate_YUV4MPEG(parameters_t *param)
 		  mjpeg_info("Processing interlaced, bottom-first %s", pngname);
 #if 0 
 		  decode_jpeg_raw(jpegdata, jpegsize,
-				  Y4M_ILACE_BOTTOM_FIRST, /* gz: was: LAV_INTER_TOP_FIRST */
+				  Y4M_ILACE_BOTTOM_FIRST,
 				  420, param->width, param->height,
 				  yuv[0], yuv[1], yuv[2]);
 #endif
