@@ -1,5 +1,5 @@
 /*
- * $Id: y4mstabilizer.c,v 1.4 2005/05/09 22:34:09 sms00 Exp $
+ * $Id: y4mstabilizer.c,v 1.9 2007/04/01 18:06:06 sms00 Exp $
  *
  * written by J. Macropol <jm@wx.gd-ais.com>
  *	Framework and shifting code adapted from y4mshift by Steve Schultz.
@@ -8,9 +8,9 @@
  * Program to stabilize a video stream.
  * Works only non-interlaced yuv4mpeg streams for now.
  *
- * Usage: y4mstab [-v] [-a <alpha>] [-r <srchRadius>]
+ * Usage: y4mstabilizer [-v] [-a <alpha>] [-r <srchRadius>]
  *
- *	-v		Verbose.   Move -vs make it more verbose.
+ *	-v		Verbose.  Add additional -v to increase verbosity/debug
  *	-a <alpha>	The alpha value is a "viscosity" measure (see below).
  *	-r <srchRadius>	How far to look for movement.
  *	-s <stride>	How far apart the motion search points are.
@@ -22,6 +22,12 @@
  * Higher values resist panning more at the expense of greater
  * shifting.  <alpha> defaults to 0.95, a fairly high value, on the theory
  * that you probably have some significant vibrations that need smoothing.
+ * Separate alpha values for the X- and Y-axes can be specified by separating
+ * them with a colon.  For example,
+ * 	-a 0.7:0.9
+ * would set the X-axis alpha to 0.7, and the Y-axis alpha to 0.9.  Thus,
+ * the stabilizer would be much more responsive to side-to-side movement,
+ * while resisting up-and-down motion.
  *
  * The <srchRadius> defaults to 15.  Smaller values speed things up,
  * but won't be able to cope with large/fast movements as well.
@@ -60,7 +66,8 @@ struct
     int		nosuper;	/* Flag not to supersample chroma on shift */
     int		rad, diam;	/* Search radius and diameter */
     int		stride;		/* Stride between motion points */
-    float	alpha;		/* Viscosity */
+    float	alphax;		/* X Viscosity */
+    float	alphay;		/* Y Viscosity */
     float	gsX, gsY;	/* Accumulated shift */
     int		ss_h, ss_v;	/* UV decimation factors */
     } Stab;
@@ -96,7 +103,7 @@ main (int argc, char **argv)
 
     Stab.rad = 15;		/* Default search radius */
     Stab.stride = 48;		/* Default stride between motion points */
-    Stab.alpha = 0.95;		/* Default viscosity */
+    Stab.alphax = Stab.alphay = 0.95; /* Default viscosity */
 
     y4m_accept_extensions(1);
 
@@ -117,7 +124,17 @@ main (int argc, char **argv)
 	    Stab.stride = atoi(optarg);
 	    break;
 	  case 'a':
-	    Stab.alpha = atof(optarg);
+	    if (strchr(optarg, ':'))
+		{
+		if (sscanf(optarg, "%g:%g", &Stab.alphax, &Stab.alphay) != 2)
+		    usage();
+		}
+	    else
+		{
+		if (sscanf(optarg, "%g", &Stab.alphax) != 1)
+		    usage();
+		Stab.alphay = Stab.alphax;
+		}
 	    break;
 	  case 'r':
 	    Stab.rad = atoi(optarg);
@@ -164,7 +181,7 @@ main (int argc, char **argv)
 	 mjpeg_error_exit1("444ALPHA (4 plane) chroma not supported!\n");
       default:
 	if (!Stab.nosuper)
-	    mjpeg_log(LOG_INFO, "Cannot supersample %s chroma",
+	    mjpeg_info("Cannot supersample %s chroma",
 		y4m_chroma_description(chroma_ss));
 	Stab.nosuper = 1;
 	break;
@@ -172,7 +189,7 @@ main (int argc, char **argv)
     width = y4m_si_get_width(&istream);
     height = y4m_si_get_height(&istream);
     if (Stab.verbose)
-	y4m_log_stream_info(LOG_INFO, "", &istream);
+	y4m_log_stream_info(mjpeg_loglev_t("info"), "", &istream);
 
     /* Initialize output stream */
     y4m_init_stream_info(&ostream);
@@ -200,7 +217,7 @@ main (int argc, char **argv)
     for (; y4m_read_frame(fdin,&istream,&iframe,yuv1) == Y4M_OK; frames++)
 	{
 	if ((Stab.verbose > 1) || (Stab.verbose && ((frames % 100) == 0)))
-	    mjpeg_log(LOG_INFO, "Frame %d", frames);
+	    mjpeg_info("Frame %d", frames);
 	subsample(yuv1[0], yuv1[3], width, height);
 	subsample(yuv1[3], yuv1[4], width/2, height/2);
 	switch (interlace)
@@ -289,9 +306,9 @@ usage (void)
 "Program to stabilize a video stream.\n"
 "Works only non-interlaced yuv4mpeg streams for now.\n"
 "\n"
-"Usage: y4mstab [-v] [-a <alpha>] [-r <srchRadius>]\n"
+"Usage: y4mstabilizer [-v] [-a <alpha>] [-r <srchRadius>]\n"
 "\n"
-"	-v		Verbose.   Move -vs make it more verbose.\n"
+"	-v		Verbose.  Repeat -v to increase verbosity/debug info\n"
 "	-a <alpha>	A \"viscosity\" measure (see below).\n"
 "	-r <srchRadius>	How far to look for movement.\n"
 "	-s <stride>	How far apart the motion search points are.\n"
@@ -303,6 +320,12 @@ usage (void)
 "Higher values resist panning more at the expense of greater\n"
 "shifting.  <alpha> defaults to 0.95, a fairly high value, on the theory\n"
 "that you probably have some significant vibrations that need smoothing.\n"
+"Separate alpha values for the X- and Y-axes can be specified by separating\n"
+"them with a colon.  For example,\n"
+"	-a 0.7:0.9\n"
+"would set the X-axis alpha to 0.7, and the Y-axis alpha to 0.9.  Thus,\n"
+"the stabilizer would be much more responsive to side-to-side movement,\n"
+"while resisting up-and-down motion.\n"
 "\n"
 "The <srchRadius> defaults to 15.  Smaller values speed things up,\n"
 "but won't be able to cope with large/fast movements as well.\n"
@@ -318,7 +341,7 @@ usage (void)
 "Very good results can be obtained with the following pipeline:\n"
 "    ... | yuvdeinterlace | \\\n"
 "	   y4mscaler -v 0 -O sar=src -O chromass=444 | \\\n"
-"	   y4mstab | \\\n"
+"	   y4mstabilizer | \\\n"
 "	   y4mscaler -v 0 -O sar=src -O chromass=420_MPEG2 | ...\n"
 , stderr);
 exit(1);
@@ -544,13 +567,13 @@ int ss_h = Stab.nosuper ? SS_H : 1;
 int ss_v = Stab.nosuper ? SS_V : 1;
 /* gmotion() returns motion in half-pixels */
 /* Factor in <alpha> the "viscosity"... */
-Stab.gsX = (Stab.gsX * Stab.alpha) + gp->x/2.0;
-Stab.gsY = (Stab.gsY * Stab.alpha) + gp->y/2.0;
+Stab.gsX = (Stab.gsX * Stab.alphax) + gp->x/2.0;
+Stab.gsY = (Stab.gsY * Stab.alphay) + gp->y/2.0;
 /* Now that we know the movement, shift to counteract it */
 shftp->x = -xround(Stab.gsX, ss_h);
 shftp->y = -xround(Stab.gsY, ss_v);
 if (Stab.verbose > 1)
-mjpeg_log(LOG_INFO, "global motion xy*2=<%d,%d>"
+mjpeg_info("global motion xy*2=<%d,%d>"
 " Accumulated xy=<%g,%g> shift xy=%d,%d>\n",
 gp->x, gp->y, Stab.gsX, Stab.gsY, shftp->x, shftp->y);
 }
