@@ -34,22 +34,6 @@
 #include "jpegutils.h"
 #include "lav_io.h"
 
-
-/* max_file_size stuff */
-#define MAX_MBYTES_PER_FILE_64 ((0x3fffff) * 93/100)        /* 4Tb.  (Most filesystems have  */
-                                                            /* fundamental limitations around ~Tb) */
-#define MAX_MBYTES_PER_FILE_32 ((0x7fffffff >> 20) * 85/100)/* Is less than 2^31 and 2*10^9 */
-#if _FILE_OFFSET_BITS == 64
-#define MAX_MBYTES_PER_FILE MAX_MBYTES_PER_FILE_64
-#else
-#define MAX_MBYTES_PER_FILE MAX_MBYTES_PER_FILE_32
-                                  /* Maximum number of Mbytes per file.
-                                     We make a conservative guess since we
-                                     only count the number of bytes output
-                                     and don't know how big the control
-                                     information will be */
-#endif
-
 #define FOURCC(a,b,c,d) ( (d<<24) | ((c&0xff)<<16) | ((b&0xff)<<8) | (a&0xff) )
 
 #define FOURCC_RIFF     FOURCC ('R', 'I', 'F', 'F')
@@ -59,7 +43,7 @@
 
 
 static int   param_quality = 80;
-static char  param_format = 'a';
+static char  param_format = 'x'; /* a format we don't know */
 static char *param_output = 0;
 static char *param_inputwav = NULL;
 static int   param_bufsize = 256*1024; /* 256 kBytes */
@@ -91,7 +75,8 @@ static void usage(void)
 	  "   -I num      force output interlacing 0:no 1:top 2:bottom field first\n"
 	  "   -q num      JPEG encoding quality [%d%%]\n"
 	  "   -b num      size of MJPEG buffer [%d kB]\n"
-          "   -m num      maimum size per file [%d MB]\n"
+          "   -m num      maximum size per file [%d MB]\n"
+	  "   -w file     WAVE file - audio data to be added to output file\n"
 	  "   -o file     output mjpeg file (REQUIRED!)\n",
 	  param_format, param_quality, param_bufsize/1024, param_maxfilesize);
 }
@@ -125,6 +110,7 @@ int main(int argc, char *argv[])
    lav_file_t *output = 0;
    int filenum = 0;
    unsigned long filesize_cur = 0;
+	char *dotptr;
    
    uint8_t *jpeg;
    int   jpegsize = 0;
@@ -133,6 +119,7 @@ int main(int argc, char *argv[])
    y4m_frame_info_t frameinfo;
    y4m_stream_info_t streaminfo;
 
+
    while ((n = getopt(argc, argv, "v:f:I:q:b:m:o:w:")) != -1) {
       switch (n) {
       case 'v':
@@ -200,19 +187,19 @@ int main(int argc, char *argv[])
       usage ();
       exit (1);
    }
-   if (param_interlace == 2 &&
-       (param_format == 'q' || param_format == 'm')) {
+   if (param_interlace == 2 && param_format == 'q') {
       mjpeg_error("cannot use -I 2 with -f %c", param_format);
       usage ();
       exit (1);
    }
+
+/*
+ * If NO limit was explicitly specified (to limit files to 2GB or less) then
+ * allow unlimited (well, ok - 4TB ;)) size.  ODML extensions will handle the
+ * AVI files and Quicktime has had 64bit filesizes for a long time
+*/
    if (param_maxfilesize <= 0)
-   {
-      if (param_format=='a'||param_format=='A')
-         param_maxfilesize = MAX_MBYTES_PER_FILE_32;
-      else
-         param_maxfilesize = MAX_MBYTES_PER_FILE;
-   }
+      param_maxfilesize = MAX_MBYTES_PER_FILE_64;
 
    (void)mjpeg_default_handler_verbosity(verbose);   
    fd_in = 0;                   /* stdin */
@@ -251,9 +238,28 @@ int main(int argc, char *argv[])
    if (param_interlace < 0) {
        param_interlace = y4m_si_get_interlace(&streaminfo);
     }
-   if (param_interlace == LAV_INTER_TOP_FIRST  && param_format == 'A')
+
+/* If no desired format option was given, we try to detect the format with */
+/* the last 4 char of the filename */
+#ifdef HAVE_LIBQUICKTIME 
+	dotptr = strrchr(param_output, '.');
+	if ( (!strcasecmp(dotptr+1, "mov")) && (param_format == 'x') )
+		param_format = 'q';
+#endif
+	if ( (!strcasecmp(dotptr+1, "avi")) && (param_format == 'x') )
+		param_format = 'a';
+
+/* Telling the people which format we really use */
+	if (param_format == 'a')
+		mjpeg_info("creating AVI output format");
+	else if (param_format == 'q')
+		mjpeg_info("creating Quicktime output format");
+	else 
+		mjpeg_error_exit1("No format specified add the -f option");
+
+   if (param_interlace == Y4M_ILACE_TOP_FIRST  && param_format == 'A')
       param_format = 'a';
-   else if (param_interlace == LAV_INTER_BOTTOM_FIRST && param_format == 'a')
+   else if (param_interlace == Y4M_ILACE_BOTTOM_FIRST && param_format == 'a')
       param_format = 'A';
    fps = Y4M_RATIO_DBL(y4m_si_get_framerate(&streaminfo));
 
@@ -379,8 +385,7 @@ int main(int argc, char *argv[])
       fflush (stdout);
       jpegsize = encode_jpeg_raw (jpeg, param_bufsize, param_quality,
                                   param_interlace,
-				  ((y4m_si_get_chroma(&streaminfo) == Y4M_CHROMA_422)? CHROMA422:
-				   CHROMA420),
+				  y4m_si_get_chroma(&streaminfo),
                                   y4m_si_get_width(&streaminfo),
 				  y4m_si_get_height(&streaminfo),
                                   yuv[0], yuv[1], yuv[2]);
