@@ -1,4 +1,4 @@
-/* encoderparams - class holding all the various control parameters for 
+/* encoderparams - class holding all the various control parameters for
    and individual encoder instance.  For speed a lot of address offsets/sizes
    are computed once-and-for-all and held in this object.
 */
@@ -66,53 +66,37 @@
 
 EncoderParams::EncoderParams( const MPEG2EncOptions &encoptions)
 {
-    memset( this, -1, sizeof(EncoderParams) ); // REMOVE
 }
 
 void EncoderParams::InitEncodingControls( const MPEG2EncOptions &options)
 {
 
+    coding_tolerance = 0.1;
 	/* Tune threading and motion compensation for specified number of CPU's 
 	   and specified speed parameters.
 	 */
     
-	act_boost =  options.act_boost >= 0.0 
-        ? (options.act_boost+1.0)
-        : (options.act_boost-1.0);
+	act_boost =  1.0+options.act_boost;
     boost_var_ceil = options.boost_var_ceil;
 	switch( options.num_cpus )
 	{
 
-	case 0 : /* Special case for debugging... turns of multi-threading */
-		max_encoding_frames = 1;
-		refine_from_rec = true;
-		parallel_read = false;
+	case 0 : /* Special case for debugging... turns of all multi-threading */
+		encoding_parallelism = 0;
 		break;
-
-	case 1 :
-		max_encoding_frames = 1;
-		refine_from_rec = true;
-		parallel_read = true;
+  case 1 : /* Currently this is the default option */
+		encoding_parallelism = 1;
 		break;
 	case 2:
-		max_encoding_frames = 2;
-		refine_from_rec = true;
-		parallel_read = true;
+		encoding_parallelism = 2;
 		break;
 	default :
-		max_encoding_frames = options.num_cpus > MAX_WORKER_THREADS-1 ?
+		encoding_parallelism = options.num_cpus > MAX_WORKER_THREADS-1 ?
 			                  MAX_WORKER_THREADS-1 :
 			                  options.num_cpus;
-		refine_from_rec = false;
-		parallel_read = true;
 		break;
 	}
 
-    max_active_ref_frames = 
-        M == 0 ? max_encoding_frames : (max_encoding_frames+2);
-    max_active_b_frames = 
-        M <= 1 ? 0 : max_encoding_frames+1;
-
 	me44_red		= options.me44_red;
 	me22_red		= options.me22_red;
 
@@ -128,7 +112,7 @@ void EncoderParams::InitEncodingControls( const MPEG2EncOptions &options)
 #ifdef DEBUG_MOTION_EST
     static const int MARGIN = 64;
 #else
-    static const int MARGIN = 0;
+    static const int MARGIN = 8;
 #endif
     
 #ifdef HAVE_ALTIVEC
@@ -199,7 +183,7 @@ static int f_code( int max_radius )
 void EncoderParams::Init( const MPEG2EncOptions &options )
 {
 	int i;
-    const char *msg;
+    const char *msg = 0;
 
 	//istrm_nframes = 999999999; /* determined by EOF of stdin */
 
@@ -214,9 +198,14 @@ void EncoderParams::Init( const MPEG2EncOptions &options )
 	M_min = options.preserve_B ? M : 1;
 	if( M > N_max )
 		M = N_max;
-	mpeg1           = (options.mpeg == 1);
+	mpeg1       = (options.mpeg == 1);
 	fieldpic        = (options.fieldenc == 2);
+    dualprime   = options.hack_dualprime == 1 && M == 1;
 
+    pulldown_32     = options.vid32_pulldown;
+
+    aspectratio     = options.aspect_ratio;
+    frame_rate_code = options.frame_rate;
     // SVCD and probably DVD? mandate progressive_sequence = 0 
     switch( options.format )
     {
@@ -227,14 +216,21 @@ void EncoderParams::Init( const MPEG2EncOptions &options )
     case MPEG_FORMAT_DVD_NAV :
         prog_seq = 0;
         break;
+    case MPEG_FORMAT_ATSC1080i :
+    case MPEG_FORMAT_ATSC480i :
+        prog_seq = frame_rate_code == 1 || frame_rate_code == 2 ? 0 : 1;
+        break;
+    case MPEG_FORMAT_ATSC720p :
+    case MPEG_FORMAT_ATSC480p :
+        prog_seq = 1;
+        break;
     default :
-        prog_seq        = (options.mpeg == 1 || options.fieldenc == 0);
+        // If we want 3:2 pulldown must code prog_seq as otherwise
+        // repeat_first_field and topfirst encode frame repetitions!!!
+        prog_seq        = (options.mpeg == 1 || (options.fieldenc == 0 && !options.vid32_pulldown));
         break;
     }
-	pulldown_32     = options.vid32_pulldown;
 
-	aspectratio     = options.aspect_ratio;
-	frame_rate_code = options.frame_rate;
 	dctsatlim		= mpeg1 ? 255 : 2047;
 
 	/* If we're using a non standard (VCD?) profile bit-rate adjust	the vbv
@@ -285,39 +281,85 @@ void EncoderParams::Init( const MPEG2EncOptions &options )
 	seq_length_limit = options.seq_length_limit;
 	nonvid_bit_rate = options.nonvid_bitrate * 1000;
 	low_delay       = 0;
-	constrparms     = (options.mpeg == 1 && 
+	constrparms     = (options.mpeg == 1 &&
 						   !MPEG_STILLS_FORMAT(options.format));
-	profile         = 4; /* Main profile resp. */
-	level           = 8; /* Main Level      CCIR 601 rates */
-	switch(options.norm)
-	{
-	case 'p': video_format = 1; break;
-	case 'n': video_format = 2; break;
-	case 's': video_format = 3; break;
-	default:  video_format = 5; break; /* unspec. */
-	}
-	switch(options.norm)
-	{
-	case 's':
-	case 'p':  /* ITU BT.470  B,G */
-		color_primaries = 5;
-		transfer_characteristics = 5; /* Gamma = 2.8 (!!) */
-		matrix_coefficients = 5; 
-        msg = "PAL B/G";
-		break;
-	case 'n': /* SMPTPE 170M "modern NTSC" */
-		color_primaries = 6;
-		matrix_coefficients = 6; 
-		transfer_characteristics = 6;
-        msg = "NTSC";
-		break; 
-	default:   /* unspec. */
-		color_primaries = 2;
-		matrix_coefficients = 2; 
-		transfer_characteristics = 2;
-        msg = "unspecified";
-		break;
-	}
+	profile         = MAIN_PROFILE;
+    
+    level = options.level;
+    // Force appropriate level for standards-compliant preset-formats
+    switch(options.format)
+    {
+        case MPEG_FORMAT_ATSC720p :
+        case MPEG_FORMAT_ATSC1080i :
+            level = HIGH_LEVEL;
+            break;
+        case MPEG_FORMAT_SVCD_STILL :
+        case MPEG_FORMAT_SVCD :
+        case MPEG_FORMAT_DVD :
+        case MPEG_FORMAT_DVD_NAV :
+        case MPEG_FORMAT_VCD :
+        default :
+            level = MAIN_LEVEL;
+            break;
+    };
+    if( level == 0 )
+        level = MAIN_LEVEL;
+
+    if( MPEG_SDTV_FORMAT(options.format) )
+    {
+        switch(options.norm)
+        {
+        case 'p': video_format = 1; break;
+        case 'n': video_format = 2; break;
+        case 's': video_format = 3; break;
+        default:  video_format = 5; break; /* unspec. */
+        }
+     
+        switch(options.norm)
+        {
+        case 's':
+        case 'p':  /* ITU BT.470  B,G */
+            color_primaries = 5;
+            transfer_characteristics = 5; /* Gamma = 2.8 (!!) */
+            matrix_coefficients = 5; 
+            msg = "PAL B/G";
+            break;
+        case 'n': /* SMPTPE 170M "modern NTSC" */
+            color_primaries = 6;
+            matrix_coefficients = 6; 
+            transfer_characteristics = 6;
+            msg = "NTSC";
+            break; 
+        default:   /* unspec. */
+            color_primaries = 2;
+            matrix_coefficients = 2; 
+            transfer_characteristics = 2;
+            msg = "unspecified";
+            break;
+        }
+    }
+    else
+    {
+        video_format = 0; // 'Component'
+        switch( options.format )
+        {
+            case MPEG_FORMAT_ATSC480i : /* SMPTPE 170M "modern NTSC" */
+            case MPEG_FORMAT_ATSC480p :
+                color_primaries = 6;
+                matrix_coefficients = 6; 
+                transfer_characteristics = 6;
+                break;
+            case MPEG_FORMAT_ATSC720p :/* ITU.R BT.709 HDTV */
+            case MPEG_FORMAT_ATSC1080i :
+                color_primaries = 1;
+                matrix_coefficients = 1; 
+                transfer_characteristics = 1;
+                break;
+            default : 
+                abort();
+                
+        };
+    };
     mjpeg_info( "Setting colour/gamma parameters to \"%s\"", msg);
 
     horizontal_size = options.in_img_width;
@@ -344,9 +386,22 @@ void EncoderParams::Init( const MPEG2EncOptions &options )
             display_vertical_size    = options.in_img_height;
         }
 		break;
-	default:
-		display_horizontal_size  =  options.in_img_width;
-		display_vertical_size    =  options.in_img_height;
+  
+      // ATSC 1080i is unusual in that it *requires* display of 1080 lines
+      // when 1088 are coded 
+     case MPEG_FORMAT_ATSC1080i :
+         display_vertical_size = 1080;
+         break;
+     
+     default:
+        if( options.display_hsize <= 0 )
+            display_horizontal_size  = options.in_img_width;
+        else
+            display_horizontal_size = options.display_hsize;
+        if( options.display_vsize <= 0 )
+		  display_vertical_size = options.in_img_height;
+        else
+            display_vertical_size = options.display_vsize;
 		break;
 	}
 
@@ -364,8 +419,8 @@ void EncoderParams::Init( const MPEG2EncOptions &options )
 		else
 			fieldorder = options.input_interlacing;
 
-		topfirst = (fieldorder == Y4M_ILACE_TOP_FIRST || 
-                              fieldorder ==Y4M_ILACE_NONE );
+		topfirst = 
+            (fieldorder == Y4M_ILACE_TOP_FIRST || fieldorder ==Y4M_ILACE_NONE );
 	}
 	else
 		topfirst = 0;
@@ -383,7 +438,7 @@ void EncoderParams::Init( const MPEG2EncOptions &options )
 	qscale_tab[0] 
 		= qscale_tab[1] 
 		= qscale_tab[2] 
-		= options.mpeg == 1 ? 0 : 1;
+		= options.mpeg == 1 ? 0 : 0;
 
 	intravlc_tab[0] 
 		= intravlc_tab[1] 
@@ -583,6 +638,7 @@ void EncoderParams::Init( const MPEG2EncOptions &options )
 		mjpeg_warn( "Frame height won't split into two equal field pictures...");
 		mjpeg_warn( "forcing encoding as progressive video");
 		prog_seq = 1;
+        pulldown_32 = false;
 		fieldpic = 0;
 	}
 
@@ -594,7 +650,7 @@ void EncoderParams::Init( const MPEG2EncOptions &options )
 	}
 
 
-	if (prog_seq && topfirst)
+	if (prog_seq && topfirst )
 	{
 		mjpeg_info("prog sequence setting top_field_first = 0");
 		topfirst = 0;
@@ -641,6 +697,8 @@ void EncoderParams::Init( const MPEG2EncOptions &options )
 
     InitQuantMatrices( options );
     InitEncodingControls( options );
+
+    chapter_points.insert(chapter_points.end(),options.chapter_points.begin(),options.chapter_points.end());
 }
 
 
