@@ -20,6 +20,8 @@
  */
 
 #include <stdio.h>
+#include <cassert>
+#include <limits.h>
 
 #include "macroblock.hh"
 #include "mpeg2syntaxcodes.h"
@@ -27,12 +29,17 @@
 
 void MacroBlock::Encode()
 { 
-    MotionEstimate();
-    SelectCodingModeOnVariance();
     Predict();
     Transform();
 }
 
+
+void MacroBlock::MotionEstimateAndModeSelect()
+{ 
+    MotionEstimate();
+    SelectCodingModeOnVariance();
+}
+
 void MacroBlock::MotionEstimate()
 {
 	if (picture->pict_struct==FRAME_PICTURE)
@@ -50,23 +57,58 @@ void MacroBlock::SelectCodingModeOnVariance()
     vector<MotionEst>::iterator i;
     vector<MotionEst>::iterator min_me;
     int best_score = INT_MAX;
+    int best_fwd_score = INT_MAX;
     int cur_score;
 
+
+    assert( best_of_kind_me.begin()->mb_type == MB_INTRA );
     //
     // Select motion estimate with lowest variance
     // Penalise the INTRA motion type slightly because it can't be
-    // skip coded.
+    // skip coded and the DC coefficient is usually large...
     for( i = best_of_kind_me.begin(); i < best_of_kind_me.end(); ++ i)
     {
-        cur_score = i->var + (i->mb_type == MB_INTRA ? 4*4*256 : 0);
+        cur_score = i->var + (i->mb_type == MB_INTRA ? 3*3*256 : 0);
         if( cur_score < best_score )
         {
             best_score = cur_score;
-            min_me = i;
+            best_me = &*i;
+        }
+        if( i->mb_type & MB_BACKWARD == 0 && cur_score < best_fwd_score)
+        {   
+            best_fwd_score = cur_score;
+            best_fwd_me = &*i;
         }
     }
-    final_me = *min_me;
-} 
+
+}
+
+/**********************************************
+ *
+ * ForceIFrame - Force selection of intra-coding so that that macroblock
+ *             can be correctly coded in an I Frame.
+ *
+ *********************************************/
+
+void MacroBlock::ForceIFrame()
+{
+    vector<MotionEst>::iterator i = best_of_kind_me.begin();
+    assert( i->mb_type == MB_INTRA );
+    best_me = &*i;
+}
+
+/**********************************************
+ *
+ * ForcePFrame - Force selection of motion-estimation so that that macroblock
+ *             can be correctly coded in an P Frame.  I.e. use only forward
+ *             motion estimatino.
+ *
+ *********************************************/
+
+void MacroBlock::ForcePFrame()
+{
+    best_me = best_fwd_me;
+}
 
 
 /* 
