@@ -45,7 +45,7 @@
  * 02111-1307, USA.
  *
  */
-
+
 #include <config.h>
 #include <stdio.h>
 #ifdef HAVE_GETOPT_H
@@ -56,17 +56,20 @@
 #include <stdlib.h>
 #include <fcntl.h>
 
+#include <algorithm>
+
 #include "mpeg2encoder.hh"
 #include "mpeg2encoptions.hh"
 #include "encoderparams.hh"
 #include "picturereader.hh"
+#include "imageplanes.hh"
 #include "elemstrmwriter.hh"
 #include "quantize.hh"
-#include "ratectl.hh"
+#include "ontheflyratectl.hh"
+#include "pass1ratectl.hh"
+#include "pass2ratectl.hh"
 #include "seqencoder.hh"
 #include "mpeg2coder.hh"
-
-#include "yuv4mpeg.h"
 #include "format_codes.h"
 #include "mpegconsts.h"
 
@@ -88,13 +91,36 @@
 class FILE_StrmWriter : public ElemStrmWriter
 {
 public:
-    FILE_StrmWriter( EncoderParams &encoder, const char *ofile_ptr ); 
-    virtual ~FILE_StrmWriter();       
-    void PutBits( uint32_t val, int n);
-    void FrameBegin();
-    void FrameFlush();
-    void FrameDiscard();
-    
+
+
+    FILE_StrmWriter( EncoderParams &encparams, const char *outfilename ) 
+        {
+            /* open output file */
+            if (!(outfile=fopen(outfilename,"wb")))
+            {
+                mjpeg_error_exit1("Couldn't create output file %s",outfilename);
+            }
+        }
+
+    virtual void WriteOutBufferUpto( const uint8_t *buffer, const uint32_t flush_upto )
+        {
+            size_t written = fwrite( buffer, 
+                                     sizeof(uint8_t), 
+                                     static_cast<size_t>(flush_upto),
+                                     outfile );
+            if( written != static_cast<size_t>(flush_upto) )
+            {
+                mjpeg_error_exit1( strerror(ferror(outfile)) );
+            }
+	       flushed += flush_upto;
+        }
+
+    virtual ~FILE_StrmWriter()
+        {
+            fclose( outfile );
+        }
+        
+    virtual uint64_t BitCount() { return flushed * 8LL; }
 private:
     FILE *outfile;
 };
@@ -102,77 +128,29 @@ private:
 
 
 
-FILE_StrmWriter::FILE_StrmWriter( EncoderParams &encparams, const char *outfilename ) :
-	ElemStrmWriter( encparams )
-{
-	/* open output file */
-	if (!(outfile=fopen(outfilename,"wb")))
-	{
-		mjpeg_error_exit1("Couldn't create output file %s",outfilename);
-	}
-}
-
-FILE_StrmWriter::~FILE_StrmWriter()
-{
-	fclose( outfile );
-}
-
-/**************
- *
- * Write rightmost n (0<=n<=32) bits of val to outfile 
- *
- *************/
-void FILE_StrmWriter::PutBits(uint32_t val, int n)
-{
-	val = (n == 32) ? val : (val & (~(0xffffffffU << n)));
-	while( n >= outcnt )
-	{
-		outbfr = (outbfr << outcnt ) | (val >> (n-outcnt));
-		putc( outbfr, outfile );
-		n -= outcnt;
-		outcnt = 8;
-		++bytecnt;
-	}
-	if( n != 0 )
-	{
-		outbfr = (outbfr<<n) | val;
-		outcnt -= n;
-	}
-}
-
-
-void FILE_StrmWriter::FrameBegin()
-{
-}
-
-void FILE_StrmWriter::FrameFlush()
-{
-}
-
-void FILE_StrmWriter::FrameDiscard()
-{
-}
-
-
 /**************************
  *
- * Derived class for File (actually: pipe and FIFO) based bitstream output
+ * Derived class for File (actually: pipe and FIFO) based input of frames
+ * in the YUV4MPEG2 format
  *
  *************************/
 
-
 class Y4MPipeReader : public PictureReader
 {
 public:
-    Y4MPipeReader( EncoderParams &encparams, int pipe_fd );
-    ~Y4MPipeReader() {}
+
+    Y4MPipeReader( EncoderParams &encparams, int istrm_fd );
+    ~Y4MPipeReader();
+
     void StreamPictureParams( MPEG2EncInVidParams &strm );
 protected:
-    bool LoadFrame( );
+    bool LoadFrame( ImagePlanes &image );
 private:
     int PipeRead(  uint8_t *buf, int len);
 
     int pipe_fd;
+    y4m_stream_info_t _si;
+    y4m_frame_info_t _fi;
 };
  
 
@@ -180,6 +158,15 @@ Y4MPipeReader::Y4MPipeReader( EncoderParams &encparams, int istrm_fd ) :
     PictureReader( encparams ),
     pipe_fd( istrm_fd )
 {
+    y4m_init_stream_info(&_si);
+    y4m_init_frame_info(&_fi);
+}
+
+
+Y4MPipeReader::~Y4MPipeReader()
+{
+    y4m_fini_stream_info(&_si);
+    y4m_fini_frame_info(&_fi);
 }
 
 
@@ -198,24 +185,20 @@ void Y4MPipeReader::StreamPictureParams( MPEG2EncInVidParams &strm )
 {
    int n;
    y4m_ratio_t sar;
-   y4m_stream_info_t si;
 
-   y4m_init_stream_info (&si);  
-   if ((n = y4m_read_stream_header (pipe_fd, &si)) != Y4M_OK) {
-       mjpeg_log( LOG_ERROR, 
-                  "Could not read YUV4MPEG2 header: %s!",
-                  y4m_strerr(n));
+   if ((n = y4m_read_stream_header (pipe_fd, &_si)) != Y4M_OK) {
+       mjpeg_error("Could not read YUV4MPEG2 header: %s!", y4m_strerr(n));
       exit (1);
    }
 
-   strm.horizontal_size = y4m_si_get_width(&si);
-   strm.vertical_size = y4m_si_get_height(&si);
-   strm.frame_rate_code = mpeg_framerate_code(y4m_si_get_framerate(&si));
-   strm.interlacing_code = y4m_si_get_interlace(&si);
+   strm.horizontal_size = y4m_si_get_width(&_si);
+   strm.vertical_size = y4m_si_get_height(&_si);
+   strm.frame_rate_code = mpeg_framerate_code(y4m_si_get_framerate(&_si));
+   strm.interlacing_code = y4m_si_get_interlace(&_si);
 
    /* Deduce MPEG aspect ratio from stream's frame size and SAR...
       (always as an MPEG-2 code; that's what caller expects). */
-   sar = y4m_si_get_sampleaspect(&si);
+   sar = y4m_si_get_sampleaspect(&_si);
    strm.aspect_ratio_code = 
        mpeg_guess_mpeg_aspect_code(2, sar, 
                                    strm.horizontal_size, 
@@ -238,21 +221,16 @@ void Y4MPipeReader::StreamPictureParams( MPEG2EncInVidParams &strm )
  *
  ****************************/
 
-bool Y4MPipeReader::LoadFrame( )
+bool Y4MPipeReader::LoadFrame( ImagePlanes &image )
 {
-   y4m_frame_info_t fi;
    int h,v,y;
-   y4m_init_frame_info (&fi);
-   int buffer_slot = frames_read % input_imgs_buf_size;
 
 
-   if ((y = y4m_read_frame_header (pipe_fd, &fi)) != Y4M_OK) 
+   if ((y = y4m_read_frame_header (pipe_fd, &_si, &_fi)) != Y4M_OK) 
    {
        if( y != Y4M_ERR_EOF )
-           mjpeg_log (LOG_WARN, 
-                      "Error reading frame header (%d): code%s!", 
-                      frames_read,
-                      y4m_strerr (y));
+           mjpeg_warn("Error reading frame header (%d): code%s!", 
+                      frames_read, y4m_strerr (y));
        return true;
       }
       
@@ -261,20 +239,20 @@ bool Y4MPipeReader::LoadFrame( )
    int i;
    for(i=0;i<v;i++)
    {
-       if( PipeRead(input_imgs_buf[buffer_slot][0]+i*encparams.phy_width,h)!=h)
+       if( PipeRead(image.Plane(0)+i*encparams.phy_width,h)!=h)
            return true;
    }
-   lum_mean[buffer_slot] = LumMean(input_imgs_buf[buffer_slot][0] );
+
    v = encparams.vertical_size/2;
    h = encparams.horizontal_size/2;
    for(i=0;i<v;i++)
    {
-       if(PipeRead(input_imgs_buf[buffer_slot][1]+i*encparams.phy_chrom_width,h)!=h)
+       if(PipeRead(image.Plane(1)+i*encparams.phy_chrom_width,h)!=h)
            return true;
    }
    for(i=0;i<v;i++)
    {
-       if(PipeRead(input_imgs_buf[buffer_slot][2]+i*encparams.phy_chrom_width,h)!=h)
+       if(PipeRead(image.Plane(2)+i*encparams.phy_chrom_width,h)!=h)
            return true;
    }
    return false;
@@ -297,6 +275,8 @@ int Y4MPipeReader::PipeRead(uint8_t *buf, int len)
    return r;
 }
 
+
+
 /**************************
  *
  * Derived class for options set from command line
@@ -340,7 +320,7 @@ void MPEG2EncCmdLineOptions::DisplayFrameRates(void)
 {
  	unsigned int i;
 	printf("Frame-rate codes:\n");
-	for( i = 1; mpeg_valid_framerate_code(i); ++i )
+    for( i = 1; mpeg_valid_framerate_code(i); ++i )
 	{
 		printf( "%2d - %s\n", i, mpeg_framerate_code_definition(i));
 	}
@@ -467,10 +447,17 @@ void MPEG2EncCmdLineOptions::Usage()
 "--verbose|-v num\n" 
 "    Level of verbosity. 0 = quiet, 1 = normal 2 = verbose/debug\n"
 "--format|-f fmt\n"
-"    Set pre-defined mux format fmt.\n"
+"    Encoding profile\n"
 "    [0 = Generic MPEG1, 1 = standard VCD, 2 = user VCD,\n"
 "     3 = Generic MPEG2, 4 = standard SVCD, 5 = user SVCD,\n"
-"     6 = VCD Stills sequences, 7 = SVCD Stills sequences, 8|9 = DVD]\n"
+"     6 = VCD Stills sequences, 7 = SVCD Stills sequences, 8|9 = DVD,\n"
+"     10 = ATSC 480i, 11 = ATSC 480p, 12 = ATSC 720p, 13 = ATSC 1080i]\n"
+"--display-hsize|-x [32..16383]\n"
+"   Set the the display-horizontal-size hint in MPEG-2 output to\n"
+"   something other than the encoded image width\n"
+"--display-vsize|-y [32..16383]\n"
+"   Set the the display-vertical-size hint in MPEG-2 output to\n"
+"   something other than the encoded image height\n" 
 "--aspect|-a num\n"
 "    Set displayed image aspect ratio image (default: 2 = 4:3)\n"
 "    [1 = 1:1, 2 = 4:3, 3 = 16:9, 4 = 2.21:1]\n"
@@ -480,8 +467,10 @@ void MPEG2EncCmdLineOptions::Usage()
 "    0 = Display frame rate code table\n"
 "--video-bitrate|-b num\n"
 "    Set Bitrate of compressed video in KBit/sec\n"
-"    (default: 1152 for VCD, 2500 for SVCD, 7500 for DVD)\n"
-"--nonvideo-birate|-B num\n"
+"    (default: 1152.0 for VCD, 2500.0 for SVCD, 7500.0 for DVD)\n"
+"--ratecontroller|-A [0..1] (default:0)\n"
+"    Specify ratecontrol alorithm\n"
+"--nonvideo-bitrate|-B num\n"
 "    Non-video data bitrate to assume for sequence splitting\n"
 "    calculations (see also --sequence-length).\n"
 "--quantisation|-q num\n"
@@ -517,7 +506,7 @@ void MPEG2EncCmdLineOptions::Usage()
 "    Preserve two B frames between I/P frames when placing GOP boundaries\n"
 "--quantisation-reduction|-Q num\n"
 "    Max. quantisation reduction for highly active blocks\n"
-"    [0.0 .. 5] (default: 0.0)\n"
+"    [0.0 .. 4.0] (default: 0.0)\n"
 "--quant-reduction-max-var|-X num\n"
 "    Luma variance below which quantisation boost (-Q) is used\n"
 "    [0.0 .. 2500.0](default: 0.0)\n"
@@ -550,16 +539,18 @@ void MPEG2EncCmdLineOptions::Usage()
 "    Force setting of playback field order to bottom or top first\n"
 "--multi-thread|-M num\n"
 "    Activate multi-threading to optimise throughput on a system with num CPU's\n"
-"    [0..32], 0=no multithreading, (default: 1)\n"
+"    [0..32], 0=no multithreading, (default: 0)\n"
 "--correct-svcd-hds|-C\n"
 "    Force SVCD horizontal_display_size to be 480 - standards say 540 or 720\n"
 "    But many DVD/SVCD players screw up with these values.\n"
-"--no-altscan-mpeg2\n"
-"    Force MPEG2 *not* to use alternate block scanning.  This may allow some\n"
-"    buggy players to play SVCD streams\n"
 "--no-constraints\n"
-"    Deactivate the constraints for maximum video resolution and sample rate.\n"
+"    Deactivate constraints for maximum video resolution and sample rate.\n"
 "    Could expose bugs in the software at very high resolutions!\n"
+"--no-altscan-mpeg2\n"
+"    Deactivate the use of the alternate block pattern for MPEG-2.  This is\n"
+"    A work-around for a Bug in an obscure hardware decoder.\n"
+"--dualprime-mpeg2\n"
+"    Turn ON use of dual-prime motion compensation. Default is OFF unless this option is used\n"
 "--custom-quant-matrices|-K kvcd|tmpgenc|default|hi-res|file=inputfile|help\n"
 "    Request custom or userspecified (from a file) quantization matrices\n"
 "--unit-coeff-elim|-E num\n"
@@ -569,6 +560,11 @@ void MPEG2EncCmdLineOptions::Usage()
 "    coefficients are included.  Reasonable values -40 to 40\n"
 "--b-per-refframe| -R 0|1|2\n"
 "    The number of B frames to generate between each I/P frame\n"
+"--cbr|-u\n"
+"    For MPEG-2 force the use of (suboptimal) ConstantBitRate (CBR) encoding\n"
+"--chapters X[,Y[,...]]\n"
+"    Specifies which frames should be chapter points (first frame is 0)\n"
+"    Chapter points are I frames on closed GOP's.\n"
 "--help|-?\n"
 "    Print this lot out!\n"
 	);
@@ -610,21 +606,31 @@ void MPEG2EncCmdLineOptions::StartupBanner()
 		mjpeg_info( "Sequence unlimited length" );
 
 	mjpeg_info("Search radius: %d",searchrad);
+	if (mpeg == 2)
+           {
+           mjpeg_info("DualPrime: %s", hack_dualprime == 1 ? "yes" : "no");
+           }
 }
 
 
-
-
 int MPEG2EncCmdLineOptions::SetFromCmdLine( int argc,	char *argv[] )
 {
-    static const char	short_options[]=
-        "m:a:f:n:b:z:T:B:q:o:S:I:r:M:4:2:Q:X:D:g:G:v:V:F:N:tpdsZHOcCPK:E:R:";
+int n;
+int nerr = 0;
+static const char   short_options[]=
+        "l:a:f:x:y:n:b:z:T:B:q:o:S:I:r:M:4:2:A:Q:X:D:g:G:v:V:F:N:pdsHcCPK:E:R:";
 
 #ifdef HAVE_GETOPT_LONG
-    static struct option long_options[]={
-        { "verbose",           1, 0, 'v' },
+
+static struct option long_options[]=
+    {
+        { "verbose",           1, 0, 'v'
+        },
         { "format",            1, 0, 'f' },
+        { "level",             1, 0, 'l' },
         { "aspect",            1, 0, 'a' },
+        { "display-hsize",     1, 0, 'x' },
+        { "display-vsize",     1, 0, 'y' },
         { "frame-rate",        1, 0, 'F' },
         { "video-bitrate",     1, 0, 'b' },
         { "nonvideo-bitrate",  1, 0, 'B' },
@@ -638,328 +644,388 @@ int MPEG2EncCmdLineOptions::SetFromCmdLine( int argc,	char *argv[] )
         { "reduction-2x2",  1, 0, '2'},
         { "min-gop-size",      1, 0, 'g'},
         { "max-gop-size",      1, 0, 'G'},
-        { "closed-gop",        1, 0, 'c'},
-        { "force-b-b-p", 0, &preserve_B, 1},
+        { "closed-gop",        0, 0, 'c'},
+        { "force-b-b-p",       0, 0, 'P'},
+        { "ratecontroller", 1, 0, 'A' },
         { "quantisation-reduction", 1, 0, 'Q' },
         { "quant-reduction-max-var", 1, 0, 'X' },
         { "video-buffer",      1, 0, 'V' },
         { "video-norm",        1, 0, 'n' },
         { "sequence-length",   1, 0, 'S' },
-        { "3-2-pulldown",      0, &vid32_pulldown, 'p'},
+        { "3-2-pulldown",      0, 0, 'p'},
         { "keep-hf",           0, 0, 'H' },
         { "reduce-hf",         1, 0, 'N' },
-        { "sequence-header-every-gop", 0, &seq_hdr_every_gop, 1},
-        { "no-dummy-svcd-SOF", 0, &svcd_scan_data, 0 },
-        { "correct-svcd-hds", 0, &hack_svcd_hds_bug, 0},
+        { "sequence-header-every-gop", 0, 0, 's'},
+        { "no-dummy-svcd-SOF", 0, 0, 'd' },
+        { "correct-svcd-hds", 0, 0, 'C'},
         { "no-constraints", 0, &ignore_constraints, 1},
         { "no-altscan-mpeg2", 0, &hack_altscan_bug, 1},
+        { "dualprime-mpeg2", 0, &hack_dualprime, 1},
         { "playback-field-order", 1, 0, 'z'},
         { "multi-thread",      1, 0, 'M' },
         { "custom-quant-matrices", 1, 0, 'K'},
         { "unit-coeff-elim",   1, 0, 'E'},
-        { "b-per-refframe",           1, 0, 'R' },
+        { "b-per-refframe",    1, 0, 'R' },
+        { "cbr",               0, 0, 'u'},
         { "help",              0, 0, '?' },
+        { "chapters",          1, 0, 256 },
         { 0,                   0, 0, 0 }
     };
 
-
-    int n;
-    int nerr = 0;
-    while( (n=getopt_long(argc,argv,short_options,long_options, NULL)) != -1 )
+while( (n=getopt_long(argc,argv,short_options,long_options, NULL)) != -1 )
 #else
-    while( (n=getopt(argc,argv,short_options)) != -1)
+while( (n=getopt(argc,argv,short_options)) != -1)
 #endif
-	{
-		switch(n) {
-        case 0 :                /* Flag setting handled by getopt-long */
+{
+    switch(n)
+    {
+    case 0 :                /* Flag setting handled by getopt-long */
+        break;
+    case 'l' :          /* MPEG-2 level */
+        if( strcmp( optarg, "high" ) == 0 || strcmp( optarg, "h" ) == 0 )
+            level = HIGH_LEVEL;
+        else if( strcmp( optarg, "main") == 0 || strcmp( optarg, "m" ) == 0 )
+            level = MAIN_LEVEL;
+        else
+        {
+            mjpeg_error( "Level must be 'main', 'm', 'high' or 'h'");
+            ++nerr;
+        }
+        break;
+    case 'b':
+        bitrate = static_cast<int>(atof(optarg)*1000);
+        if( bitrate % 400 != 0 )
+        {
+            mjpeg_warn( "MPEG bitrate must be a multiple of 400 - rounding up" );
+            bitrate = (bitrate / 400 + 1) * 400;
+        }
+        break;
+
+    case 'T' :
+        still_size = atoi(optarg)*1024;
+        if( still_size < 20*1024 || still_size > 500*1024 )
+        {
+            mjpeg_error( "-T requires arg 20..500" );
+            ++nerr;
+        }
+        break;
+    case 'B':
+        nonvid_bitrate = atoi(optarg);
+        if( nonvid_bitrate < 0 )
+        {
+            mjpeg_error("-B requires arg > 0");
+            ++nerr;
+        }
+        break;
+    case 'D':
+        mpeg2_dc_prec = atoi(optarg)-8;
+        if( mpeg2_dc_prec < 0 || mpeg2_dc_prec > 3 )
+        {
+            mjpeg_error( "-D requires arg [8..11]" );
+            ++nerr;
+        }
+        break;
+    case 'C':
+        hack_svcd_hds_bug = 0;
+        break;
+
+    case 'q':
+        quant = atoi(optarg);
+        if(quant<1 || quant>32)
+        {
+            mjpeg_error("-q option requires arg 1 .. 32");
+            ++nerr;
+        }
+        break;
+
+    case 'x' :
+        display_hsize = atoi(optarg);
+        if( display_hsize < 32 || display_hsize >= 16384 )
+        {
+            mjpeg_error( "-x option must be in range [32..16383]" );
+            ++nerr;
+        }
+        break;
+    case 'y' :
+        display_vsize = atoi(optarg);
+        if( display_vsize < 32 || display_vsize >= 16384 )
+        {
+            mjpeg_error( "-y option must be in range [32..16383]" );
+            ++nerr;
+        }
+        break;
+    case 'a' :
+        aspect_ratio = atoi(optarg);
+        if( aspect_ratio == 0 )
+            DisplayAspectRatios();
+        /* Checking has to come later once MPEG 1/2 has been selected...*/
+        if( aspect_ratio < 0 )
+        {
+            mjpeg_error( "-a option must be positive");
+            ++nerr;
+        }
+        break;
+
+    case 'F' :
+        frame_rate = atoi(optarg);
+        if( frame_rate == 0 )
+            DisplayFrameRates();
+        if( !mpeg_valid_framerate_code(frame_rate) )
+        {
+            mjpeg_error( "illegal -F value (use -F 0 to list options)" );
+            ++nerr;
+        }
+        break;
+
+    case 'o':
+        outfilename = optarg;
+        break;
+
+    case 'I':
+        fieldenc = atoi(optarg);
+        if( fieldenc < 0 || fieldenc > 2 )
+        {
+            mjpeg_error("-I option requires 0,1 or 2");
+            ++nerr;
+        }
+        break;
+
+    case 'r':
+        searchrad = atoi(optarg);
+        if(searchrad<0 || searchrad>32)
+        {
+            mjpeg_error("-r option requires arg 0 .. 32");
+            ++nerr;
+        }
+        break;
+
+    case 'M':
+        num_cpus = atoi(optarg);
+        if(num_cpus<0 || num_cpus>32)
+        {
+            mjpeg_error("-M option requires arg 0..32");
+            ++nerr;
+        }
+        break;
+
+    case '4':
+        me44_red = atoi(optarg);
+        if(me44_red<0 || me44_red>4)
+        {
+            mjpeg_error("-4 option requires arg 0..4");
+            ++nerr;
+        }
+        break;
+
+    case '2':
+        me22_red = atoi(optarg);
+        if(me22_red<0 || me22_red>4)
+        {
+            mjpeg_error("-2 option requires arg 0..4");
+            ++nerr;
+        }
+        break;
+
+    case 'v':
+        verbose = atoi(optarg);
+        if( verbose < 0 || verbose > 2 )
+            ++nerr;
+        break;
+    case 'V' :
+        video_buffer_size = atoi(optarg);
+        if(video_buffer_size<20 || video_buffer_size>4000)
+        {
+            mjpeg_error("-v option requires arg 20..4000");
+            ++nerr;
+        }
+        break;
+
+    case 'S' :
+        seq_length_limit = atoi(optarg);
+        if(seq_length_limit<1 )
+        {
+            mjpeg_error("-S option requires arg > 1");
+            ++nerr;
+        }
+        break;
+    case 'p' :
+        vid32_pulldown = 1;
+        break;
+
+    case 'z' :
+        if( strlen(optarg) != 1 || (optarg[0] != 't' && optarg[0] != 'b' ) )
+        {
+            mjpeg_error("-z option requires arg b or t" );
+            ++nerr;
+        }
+        else if( optarg[0] == 't' )
+            force_interlacing = Y4M_ILACE_TOP_FIRST;
+        else if( optarg[0] == 'b' )
+            force_interlacing = Y4M_ILACE_BOTTOM_FIRST;
+        break;
+
+    case 'f' :
+        format = atoi(optarg);
+        if( format < MPEG_FORMAT_FIRST ||
+                format > MPEG_FORMAT_LAST )
+        {
+            mjpeg_error("-f option requires arg [%d..%d]",
+                        MPEG_FORMAT_FIRST, MPEG_FORMAT_LAST);
+            ++nerr;
+        }
+
+        break;
+
+    case 'n' :
+        switch( optarg[0] )
+        {
+        case 'p' :
+        case 'n' :
+        case 's' :
+            norm = optarg[0];
             break;
-		case 'b':
-			bitrate = atoi(optarg)*1000;
-			break;
+        default :
+            mjpeg_error("-n option requires arg n or p, or s.");
+            ++nerr;
+        }
+        break;
+    case 'g' :
+        min_GOP_size = atoi(optarg);
+        break;
+    case 'G' :
+        max_GOP_size = atoi(optarg);
+        break;
+    case 'c' :
+        closed_GOPs = true;
+        break;
+    case 'P' :
+        preserve_B = true;
+        break;
+    case 'N':
+        hf_q_boost = atof(optarg);
+        if (hf_q_boost <0.0 || hf_q_boost > 2.0)
+        {
+            mjpeg_error( "-N option requires arg 0.0 .. 2.0" );
+            ++nerr;
+            hf_q_boost = 0.0;
+        }
+        if (hf_quant == 0 && hf_q_boost != 0.0)
+            hf_quant = 1;
+        break;
+    case 'H':
+        hf_quant = 2;
+        break;
+    case 'K':
+        ParseCustomOption(optarg);
+        break;
 
-		case 'T' :
-			still_size = atoi(optarg)*1024;
-			if( still_size < 20*1024 || still_size > 500*1024 )
-			{
-				mjpeg_error( "-T requires arg 20..500" );
-				++nerr;
-			}
-			break;
+    case 'u':
+        force_cbr = 1;
+        break;
 
-		case 'B':
-			nonvid_bitrate = atoi(optarg);
-			if( nonvid_bitrate < 0 )
-			{
-				mjpeg_error("-B requires arg > 0");
-				++nerr;
-			}
-			break;
-
-        case 'D':
-            mpeg2_dc_prec = atoi(optarg)-8;
-            if( mpeg2_dc_prec < 0 || mpeg2_dc_prec > 3 )
-            {
-                mjpeg_error( "-D requires arg [8..11]" );
-                ++nerr;
-            }
-            break;
-        case 'C':
-            hack_svcd_hds_bug = 0;
-            break;
-
-		case 'q':
-			quant = atoi(optarg);
-			if(quant<1 || quant>32)
-			{
-				mjpeg_error("-q option requires arg 1 .. 32");
-				++nerr;
-			}
-			break;
-
-        case 'a' :
-			aspect_ratio = atoi(optarg);
-            if( aspect_ratio == 0 )
-				DisplayAspectRatios();
-			/* Checking has to come later once MPEG 1/2 has been selected...*/
-			if( aspect_ratio < 0 )
-			{
-				mjpeg_error( "-a option must be positive");
-				++nerr;
-			}
-			break;
-
-       case 'F' :
-			frame_rate = atoi(optarg);
-            if( frame_rate == 0 )
-				DisplayFrameRates();
-			if( !mpeg_valid_framerate_code(frame_rate) )
-			{
-				mjpeg_error( "illegal -F value (use -F 0 to list options)" );
-				++nerr;
-			}
-			break;
-
-		case 'o':
-			outfilename = optarg;
-			break;
-
-		case 'I':
-			fieldenc = atoi(optarg);
-			if( fieldenc < 0 || fieldenc > 2 )
-			{
-				mjpeg_error("-I option requires 0,1 or 2");
-				++nerr;
-			}
-			break;
-
-		case 'r':
-			searchrad = atoi(optarg);
-			if(searchrad<0 || searchrad>32)
-			{
-				mjpeg_error("-r option requires arg 0 .. 32");
-				++nerr;
-			}
-			break;
-
-		case 'M':
-			num_cpus = atoi(optarg);
-			if(num_cpus<0 || num_cpus>32)
-			{
-				mjpeg_error("-M option requires arg 0..32");
-				++nerr;
-			}
-			break;
-
-		case '4':
-			me44_red = atoi(optarg);
-			if(me44_red<0 || me44_red>4)
-			{
-				mjpeg_error("-4 option requires arg 0..4");
-				++nerr;
-			}
-			break;
-			
-		case '2':
-			me22_red = atoi(optarg);
-			if(me22_red<0 || me22_red>4)
-			{
-				mjpeg_error("-2 option requires arg 0..4");
-				++nerr;
-			}
-			break;
-
-		case 'v':
-			verbose = atoi(optarg);
-			if( verbose < 0 || verbose > 2 )
-				++nerr;
-			break;
-		case 'V' :
-			video_buffer_size = atoi(optarg);
-			if(video_buffer_size<20 || video_buffer_size>4000)
-			{
-				mjpeg_error("-v option requires arg 20..4000");
-				++nerr;
-			}
-			break;
-
-		case 'S' :
-			seq_length_limit = atoi(optarg);
-			if(seq_length_limit<1 )
-			{
-				mjpeg_error("-S option requires arg > 1");
-				++nerr;
-			}
-			break;
-		case 'p' :
-			vid32_pulldown = 1;
-			break;
-
-		case 'z' :
-			if( strlen(optarg) != 1 || (optarg[0] != 't' && optarg[0] != 'b' ) )
-			{
-				mjpeg_error("-z option requires arg b or t" );
-				++nerr;
-			}
-			else if( optarg[0] == 't' )
-				force_interlacing = Y4M_ILACE_TOP_FIRST;
-			else if( optarg[0] == 'b' )
-				force_interlacing = Y4M_ILACE_BOTTOM_FIRST;
-			break;
-
-		case 'f' :
-			format = atoi(optarg);
-			if( format < MPEG_FORMAT_FIRST ||
-				format > MPEG_FORMAT_LAST )
-			{
-				mjpeg_error("-f option requires arg [%d..%d]", 
-							MPEG_FORMAT_FIRST, MPEG_FORMAT_LAST);
-				++nerr;
-			}
-				
-			break;
-
-		case 'n' :
-			switch( optarg[0] )
-			{
-			case 'p' :
-			case 'n' :
-			case 's' :
-				norm = optarg[0];
-				break;
-			default :
-				mjpeg_error("-n option requires arg n or p, or s.");
-				++nerr;
-			}
-			break;
-		case 'g' :
-			min_GOP_size = atoi(optarg);
-			break;
-		case 'G' :
-			max_GOP_size = atoi(optarg);
-			break;
-        	case 'c' :
-            		closed_GOPs = true;
-            		break;
-		case 'P' :
-			preserve_B = true;
-			break;
-		case 'N':
-            hf_q_boost = atof(optarg);
-            if (hf_q_boost <0.0 || hf_q_boost > 2.0)
-            {
-                mjpeg_error( "-N option requires arg 0.0 .. 2.0" );
-                ++nerr;
-                hf_q_boost = 0.0;
-            }
-			if (hf_quant == 0 && hf_q_boost != 0.0)
-			   hf_quant = 1;
-			break;
-		case 'H':
-			hf_quant = 2;
-            		break;
-		case 'K':
-			ParseCustomOption(optarg);
-			break;
-        case 'E':
-            unit_coeff_elim = atoi(optarg);
-            if (unit_coeff_elim < -40 || unit_coeff_elim > 40)
-            {
-                mjpeg_error( "-E option range arg -40 to 40" );
-                ++nerr;
-            }
-            break;
-        case 'R' :
-            Bgrp_size = atoi(optarg)+1;
-            if( Bgrp_size<1 || Bgrp_size>3)
-            {
-                mjpeg_error( "-R option arg 0|1|2" );
-                ++nerr;
-            }
-            break;
-		case 's' :
-			seq_hdr_every_gop = 1;
-			break;
-		case 'd' :
-			svcd_scan_data = 0;
-			break;
-		case 'Q' :
-			act_boost = atof(optarg);
-			if( act_boost <-4.0 || act_boost > 5.0)
-			{
-				mjpeg_error( "-q option requires arg -4.0 .. 5.0");
-				++nerr;
-			}
-			break;
-		case 'X' :
-			boost_var_ceil = atof(optarg);
-			if( boost_var_ceil <0 || boost_var_ceil > 50*50 )
-			{
-				mjpeg_error( "-X option requires arg 0 .. 2500" );
-				++nerr;
-			}
-			break;
-		case ':' :
-			mjpeg_error( "Missing parameter to option!" );
-		case '?':
-		default:
-			++nerr;
-		}
-	}
-
-	/* Select input stream */
-	if(optind!=argc)
-	{
-		if( optind == argc-1 )
-		{
-			istrm_fd = open( argv[optind], O_RDONLY );
-			if( istrm_fd < 0 )
-			{
-				mjpeg_error( "Unable to open: %s: ",argv[optind] );
-				perror("");
-				++nerr;
-			}
-		}
-		else
-			++nerr;
-	}
-	else
-		istrm_fd = 0; /* stdin */
-
-	if(!outfilename)
-	{
-		mjpeg_error("Output file name (-o option) is required!");
-		++nerr;
-	}
-
-    return nerr;
+    case 'E':
+        unit_coeff_elim = atoi(optarg);
+        if (unit_coeff_elim < -40 || unit_coeff_elim > 40)
+        {
+            mjpeg_error( "-E option range arg -40 to 40" );
+            ++nerr;
+        }
+        break;
+    case 'R' :
+        Bgrp_size = atoi(optarg)+1;
+        if( Bgrp_size<1 || Bgrp_size>3)
+        {
+            mjpeg_error( "-R option arg 0|1|2" );
+            ++nerr;
+        }
+        break;
+    case 's' :
+        seq_hdr_every_gop = 1;
+        break;
+    case 'd' :
+        svcd_scan_data = 0;
+        break;
+    case 'A' :
+        rate_control = atoi(optarg);
+        if( rate_control < 0 || rate_control > 1 )
+        {
+            mjpeg_error( "-A option requires arg [0,1]");
+            ++nerr;
+        }
+        break;
+    case 'Q' :
+        act_boost = atof(optarg);
+        if( act_boost < 0.0 || act_boost > 4.0)
+        {
+            mjpeg_error( "-Q option requires arg 0.0 .. 4.0");
+            ++nerr;
+        }
+        break;
+    case 'X' :
+        boost_var_ceil = atof(optarg);
+        if( boost_var_ceil <0 || boost_var_ceil > 50*50 )
+        {
+            mjpeg_error( "-X option requires arg 0 .. 2500" );
+            ++nerr;
+        }
+        break;
+    case 256: // --chapters=X
+        for( char *x=strtok(optarg,","); x; x=strtok(0,",") )
+            chapter_points.push_back(atoi(x));
+        std::sort(chapter_points.begin(),chapter_points.end());
+        break;
+    case ':' :
+        mjpeg_error( "Missing parameter to option!" );
+    case '?':
+    default:
+        ++nerr;
+    }
 }
 
+/* Select input stream */
+if(optind!=argc)
+{
+    if( optind == argc-1 )
+    {
+        istrm_fd = open( argv[optind], O_RDONLY );
+        if( istrm_fd < 0 )
+        {
+            mjpeg_error( "Unable to open: %s: ",argv[optind] );
+            perror("");
+            ++nerr;
+        }
+    }
+    else
+        ++nerr;
+}
+else
+    istrm_fd = 0; /* stdin */
+
+if(!outfilename)
+{
+    mjpeg_error("Output file name (-o option) is required!");
+    ++nerr;
+}
+
+/*
+ * Probably not necessary but err on the safe side.  If someone wants to
+ * waste space by using a Constant Bit Rate stream then disable the '-q'
+ * parameter.  Further checks for CBR are made in mpeg2encoptions.cc 
+*/
+if (force_cbr != 0)
+    quant = 0;
+
+return nerr;
+}
+
+
 
 
 class YUV4MPEGEncoder : public MPEG2Encoder
 {
 public:
     YUV4MPEGEncoder( MPEG2EncCmdLineOptions &options );
+    void Encode();
 };
 
 
@@ -976,32 +1042,56 @@ YUV4MPEGEncoder::YUV4MPEGEncoder( MPEG2EncCmdLineOptions &cmd_options ) :
 
     writer = new FILE_StrmWriter( parms, cmd_options.outfilename );
     quantizer = new Quantizer( parms );
-    coder = new MPEG2Coder( parms, *writer );
-    bitrate_controller = new OnTheFlyRateCtl( parms );
-    seqencoder = new SeqEncoder( parms, *reader, *quantizer,
-                                 *writer, *coder, *bitrate_controller);
-
     
+    if( cmd_options.rate_control == 0 )
+    {
+        mjpeg_info( "Using one-pass rate controller" );
+        pass1ratectl = new OnTheFlyPass1( parms );
+        pass2ratectl = new OnTheFlyPass2( parms );
+    }
+    else
+    {
+        mjpeg_info( "Using statistical look-ahead/two-pass rate controller" );
+#if 0
+        pass1ratectl = new VBufPass1RC( parms );
+        pass2ratectl = new XhiPass2RC( parms );
+#else
+        mjpeg_info( "Still needs updating to new interface!" );
+        abort();
+#endif
+    }
+
+    seqencoder = new SeqEncoder( parms, *reader, *quantizer,
+                                 *writer,
+                                 *pass1ratectl,
+                                 *pass2ratectl
+                                );
+
+    // This order is important! Don't change...
     parms.Init( options );
     reader->Init();
     quantizer->Init();
-    
+    seqencoder->Init();
 
 }
 
-int main( int argc,	char *argv[] )
+void YUV4MPEGEncoder::Encode( )
 {
+    seqencoder->EncodeStream();
+}
 
-	/* Set up error logging.  The initial handling level is LOG_INFO
-	 */
+int main( int argc, char *argv[] )
+{
     MPEG2EncCmdLineOptions options;
-	mjpeg_default_handler_verbosity(options.verbose);
+
+    mjpeg_default_handler_verbosity(options.verbose);
+
     if( options.SetFromCmdLine( argc, argv ) != 0 )
 		options.Usage();
-	mjpeg_default_handler_verbosity(options.verbose);
+    mjpeg_default_handler_verbosity(options.verbose);
 
     YUV4MPEGEncoder encoder( options );
-    encoder.seqencoder->Encode();
+    encoder.Encode();
 
 #ifdef OUTPUT_STAT
 	if( statfile != NULL )
@@ -1013,8 +1103,6 @@ int main( int argc,	char *argv[] )
 	return 0;
 }
 
-
-
 
 /* 
  * Local variables:
