@@ -31,9 +31,10 @@
 #include <getopt.h>
 #endif
 #include <string>
+#include <string.h>
 #include <memory>
 #include <sys/stat.h>
-#ifndef _WIN32
+#if !defined(_WIN32) || defined(__MINGW32__)
 #include <sys/param.h>
 #endif
 #include <ctype.h>
@@ -80,7 +81,7 @@ public:
     FileOutputStream( const char *filename_pat );
     virtual int  Open( );
     virtual void Close();
-    virtual off_t SegmentSize( );
+    virtual uint64_t SegmentSize( );
     virtual void NextSegment();
     virtual void Write(uint8_t *data, unsigned int len);
 
@@ -101,6 +102,7 @@ FileOutputStream::FileOutputStream( const char *name_pat )
       
 int FileOutputStream::Open()
 {
+    segment_len = 0;
 	strm = fopen( cur_filename, "wb" );
 	if( strm == NULL )
 	{
@@ -116,13 +118,10 @@ void FileOutputStream::Close()
 }
 
 
-off_t
+uint64_t
 FileOutputStream::SegmentSize()
 {
-	struct stat stb;
-    fstat(fileno(strm), &stb);
-	off_t written = stb.st_size;
-    return written;
+    return segment_len;
 }
 
 void 
@@ -144,6 +143,7 @@ FileOutputStream::NextSegment( )
 	{
 		mjpeg_error_exit1( "Could not open for writing: %s", cur_filename );
 	}
+    segment_len = 0;
 }
 
 void
@@ -153,6 +153,7 @@ FileOutputStream::Write( uint8_t *buf, unsigned int len )
     {
         mjpeg_error_exit1( "Failed write: %s", cur_filename );
     }
+    segment_len += static_cast<uint64_t>(len);
 }
 
 
@@ -262,68 +263,79 @@ private:
 };
 
 const char CmdLineMultiplexJob::short_options[] =
-    "o:b:r:O:v:f:l:s:S:p:W:L:VMh";
+        "o:i:b:r:O:v:f:l:s:S:p:W:L:R:VCMh";
 #if defined(HAVE_GETOPT_LONG)
 struct option CmdLineMultiplexJob::long_options[] = 
 {
-	{ "verbose",           1, 0, 'v' },
-	{ "format",            1, 0, 'f' },
-	{ "mux-bitrate",       1, 0, 'r' },
-	{ "video-buffer",      1, 0, 'b' },
-	{ "lpcm-params",       1, 0, 'L' },
-	{ "output",            1, 0, 'o' },
-	{ "sync-offset",    	1, 0, 'O' },
-	{ "vbr",      	        1, 0, 'V' },
-	{ "system-headers",    1, 0, 'h' },
-	{ "split-segment",     0, 0, 'M' },
-	{ "max-segment-size",  1, 0, 'S' },
-	{ "mux-limit",   	    1, 0, 'l' },
-	{ "packets-per-pack",  1, 0, 'p' },
-	{ "sector-size",       1, 0, 's' },
-	{ "workarounds", 1, 0, 'W' },
-	{ "help",              0, 0, '?' },
-	{ 0,                   0, 0, 0   }
+    { "verbose",           1, 0, 'v' },
+    { "vdr-index",         1, 0, 'i' },
+    { "format",            1, 0, 'f' },
+    { "mux-bitrate",       1, 0, 'r' },
+    { "video-buffer",      1, 0, 'b' },
+    { "lpcm-params",       1, 0, 'L' },
+    { "output",            1, 0, 'o' },
+    { "sync-offset",       1, 0, 'O' },
+    { "vbr",               0, 0, 'V' },
+    { "cbr",               0, 0, 'C' },
+    { "system-headers",    0, 0, 'h' },
+    { "ignore-seqend-markers",     0, 0, 'M' },
+    { "run-in",            1, 0, 'R' },
+    { "max-segment-size",  1, 0, 'S' },
+    { "mux-limit",          1, 0, 'l' },
+    { "packets-per-pack",  1, 0, 'p' },
+    { "sector-size",       1, 0, 's' },
+    { "workarounds", 1, 0, 'W' },
+    { "help",              0, 0, '?' },
+    { 0,                   0, 0, 0   }
 };
 #endif
 
 
 CmdLineMultiplexJob::CmdLineMultiplexJob(unsigned int argc, char *argv[]) :
 	MultiplexJob()
+
 {
     int n;
     outfile_pattern = NULL;
 
 #if defined(HAVE_GETOPT_LONG)
-	while( (n=getopt_long(argc,argv,short_options,long_options, NULL)) != -1 )
+
+    while( (n=getopt_long(argc,argv,short_options,long_options, NULL)) != -1 )
 #else
+
     while( (n=getopt(argc,argv,short_options)) != -1 )
 #endif
-	{
-		switch(n)
-		{
+
+    {
+        switch(n)
+        {
         case 0 :
             break;
-		case 'o' :
-			outfile_pattern = optarg;
-			break;
-		case 'v' :
-			verbose = atoi(optarg);
-			if( verbose < 0 || verbose > 2 )
-				Usage(argv[0]);
-			break;
+        case 'o' :
+            outfile_pattern = optarg;
+            break;
+        case 'i' :
+            vdr_index_pathname = optarg;
+            break;
+        case 'v' :
+            verbose = atoi(optarg);
+            if( verbose < 0 || verbose > 2 )
+                Usage(argv[0]);
+            break;
+        case 'V' :
+            VBR = true;
+            break;
+        case 'C' :
+            CBR = true;
+            break;
+        case 'h' :
+            always_system_headers = true;
+            break;
 
-		case 'V' :
-			VBR = true;
-			break;
-	  
-		case 'h' :
-			always_system_headers = true;
-			break;
-
-		case 'b' :
+        case 'b' :
             if( ! ParseVideoOpt( optarg ) )
             {
-                mjpeg_error( "Illegal video decoder buffer size(s): %s", 
+                mjpeg_error( "Illegal video decoder buffer size(s): %s",
                              optarg );
                 Usage(argv[0]);
             }
@@ -336,76 +348,81 @@ CmdLineMultiplexJob::CmdLineMultiplexJob(unsigned int argc, char *argv[]) :
             }
             break;
 
-		case 'r':
-			data_rate = atoi(optarg);
-			if( data_rate < 0 )
-				Usage(argv[0]);
-			/* Convert from kbit/sec (user spec) to 50B/sec units... */
-			data_rate = (( data_rate * 1000 / 8 + 49) / 50 ) * 50;
-			break;
+        case 'r':
+            data_rate = atoi(optarg);
+            if( data_rate < 0 )
+                Usage(argv[0]);
+            /* Convert from kbit/sec (user spec) to 50B/sec units... */
+            data_rate = (( data_rate * 1000 / 8 + 49) / 50 ) * 50;
+            break;
+        
+        case 'R':
+            run_in_frames = atoi(optarg);
+            if( run_in_frames < 0 || run_in_frames > 100 )
+                Usage(argv[0]);
+            break;
 
-		case 'O':
-			if( ! ParseTimeOffset(optarg) )
-			{
-				mjpeg_error( "Time offset units if specified must: ms|s|mpt" );
-				Usage(argv[0]);
-			}
-			break;
-          
-		case 'l' : 
- 			max_PTS = atoi(optarg);
-			if( max_PTS < 1  )
-				Usage(argv[0]);
-			break;
-		
-		case 'p' : 
-			packets_per_pack = atoi(optarg);
-			if( packets_per_pack < 1 || packets_per_pack > 100  )
-				Usage(argv[0]);
-			break;
-		
-	  
-		case 'f' :
-			mux_format = atoi(optarg);
-			if( mux_format < MPEG_FORMAT_MPEG1 || mux_format > MPEG_FORMAT_LAST
-                )
-				Usage(argv[0]);
-			break;
-		case 's' :
-			sector_size = atoi(optarg);
-			if( sector_size < 256 || sector_size > 16384 )
-				Usage(argv[0]);
-			break;
-		case 'S' :
-			max_segment_size = atoi(optarg);
-			if( max_segment_size < 0  )
-				Usage(argv[0]);
-			break;
-		case 'M' :
-			multifile_segment = true;
-			break;
+        case 'O':
+            if( ! ParseTimeOffset(optarg) )
+            {
+                mjpeg_error( "Time offset units if specified must: ms|s|mpt" );
+                Usage(argv[0]);
+            }
+            break;
+
+        case 'l' :
+            max_PTS = atoi(optarg);
+            if( max_PTS < 1  )
+                Usage(argv[0]);
+            break;
+
+        case 'p' :
+            packets_per_pack = atoi(optarg);
+            if( packets_per_pack < 1 || packets_per_pack > 100  )
+                Usage(argv[0]);
+            break;
+
+
+        case 'f' :
+            mux_format = atoi(optarg);
+            if( mux_format < MPEG_FORMAT_MPEG1 || mux_format > MPEG_FORMAT_LAST
+              )
+                Usage(argv[0]);
+            break;
+        case 's' :
+            sector_size = atoi(optarg);
+            if( sector_size < 256 || sector_size > 16384 )
+                Usage(argv[0]);
+            break;
+        case 'S' :
+            max_segment_size = atoi(optarg);
+            if( max_segment_size < 0  )
+                Usage(argv[0]);
+            break;
+        case 'M' :
+            multifile_segment = true;
+            break;
         case 'W' :
             if( ! ParseWorkaroundOpt( optarg ) )
             {
                 Usage(argv[0]);
             }
             break;
-		case '?' :
-		default :
-			Usage(argv[0]);
-			break;
-		}
-	}
-	if (argc - optind < 1 || outfile_pattern == NULL)
-    {	
-		Usage(argv[0]);
+        case '?' :
+        default :
+            Usage(argv[0]);
+            break;
+        }
     }
-	(void)mjpeg_default_handler_verbosity(verbose);
-	mjpeg_info( "mplex version %s (%s %s)",VERSION,MPLEX_VER,MPLEX_DATE );
+    if (argc - optind < 1 || outfile_pattern == NULL)
+    {
+        Usage(argv[0]);
+    }
+    (void)mjpeg_default_handler_verbosity(verbose);
+    mjpeg_info( "mplex version %s (%s %s)",VERSION,MPLEX_VER,MPLEX_DATE );
 
     InputStreamsFromCmdLine( argc-(optind-1), argv+optind-1);
 }
-
 /*************************************************************************
  Usage banner for the command line wrapper.
 *************************************************************************/
@@ -438,15 +455,22 @@ void CmdLineMultiplexJob::Usage(char *str)
 	"--sector-size|-s num\n"
     "  Specify sector size in bytes for generic formats [256..16384]\n"
     "--vbr|-V\n"
-    "  Multiplex variable bit-rate video\n"
+    "  Force variable bit-rate video multiplexing\n"
+    "--cbr|-C\n"
+    "  Force constant bit-rate video multiplexing\n"
+    "--run-in|-R num\n"
+    "  Force a 'run-in' of exactly num frame intervals\n"
 	"--packets-per-pack|-p num\n"
     "  Number of packets per pack generic formats [1..100]\n"
 	"--system-headers|-h\n"
     "  Create System header in every pack in generic formats\n"
 	"--max-segment-size|-S size\n"
     "  Maximum size of output file(s) in Mbyte (default: 0) (no limit)\n"
-	"--split-segment|-M\n"
-    "  Simply split a sequence across files rather than building run-out/run-in\n"
+	"--ignore-seqend-markers|-M\n"
+    "  Don't switch to a new output file if a  sequence end marker\n"
+	"  is encountered ithe input video.\n"
+    "--vdr-index|-i <vdr-index-filename>\n"
+    "  Generate a VDR index file with the output stream\n"
     "--workaround|-W workaround [, workaround ]\n"
 	"--help|-?\n"
     "  Print this lot out!\n", str);
@@ -579,7 +603,7 @@ bool CmdLineMultiplexJob::ParseTimeOffset(const char *optarg)
     char *e;
 
     f=strtod(optarg,&e);
-    if( e ) {
+    if( *e ) {
         while(isspace(*e)) e++;
         if(!strcmp(e,"ms")) persecond=1000.0;
         else if(!strcmp(e,"s")) persecond=1.0;
@@ -587,7 +611,7 @@ bool CmdLineMultiplexJob::ParseTimeOffset(const char *optarg)
 		else
 			return false;
     }
-    video_offset = lround(f*CLOCKS/(persecond));
+    video_offset = static_cast<int>(f*CLOCKS/(persecond));
 	if( video_offset < 0 )
 	{
 		audio_offset = - video_offset;
@@ -612,9 +636,13 @@ int main (int argc, char* argv[])
 {
 	CmdLineMultiplexJob job(argc,argv);
 	FileOutputStream output( job.outfile_pattern );
-	Multiplexor mux(job, output);
+    FileOutputStream *index = job.vdr_index_pathname != 0 
+                             ? new FileOutputStream( job.vdr_index_pathname ) 
+                             : 0;
+	Multiplexor mux(job, output, index );
 	mux.Multiplex();
-
+    if( index != 0 )
+        delete index;
     return (0);	
 }
 
