@@ -22,6 +22,7 @@
 #include <config.h>
 #include <math.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <mjpeg_types.h>
 #include <mjpeg_logging.h>
@@ -45,7 +46,7 @@
  *
  ***************/
 
-Multiplexor::Multiplexor(MultiplexJob &job, OutputStream &output)
+Multiplexor::Multiplexor(MultiplexJob &job, OutputStream &output, OutputStream *index)
 {
     underrun_ignore = 0;
     underruns = 0;
@@ -54,9 +55,19 @@ Multiplexor::Multiplexor(MultiplexJob &job, OutputStream &output)
     InitInputStreams(job);
 
     psstrm = new PS_Stream(mpeg, sector_size, output, max_segment_size );
-
+    vdr_index = index;
 }
 
+Multiplexor::~Multiplexor()
+{
+    delete psstrm;
+    while (!estreams.empty()) {
+        delete estreams.back();
+        estreams.pop_back();
+    }
+    vstreams.clear();
+    astreams.clear();
+}
 
 /******************************************************************
  *
@@ -73,7 +84,6 @@ void Multiplexor::InitSyntaxParameters(MultiplexJob &job)
 	seg_starts_with_video = false;
 	audio_buffer_size = 4 * 1024;
     mux_format = job.mux_format;
-    vbr = job.VBR;
     packets_per_pack = job.packets_per_pack;
     data_rate = job.data_rate;
     mpeg = job.mpeg;
@@ -82,8 +92,9 @@ void Multiplexor::InitSyntaxParameters(MultiplexJob &job)
     sector_size = job.sector_size;
 	split_at_seq_end = !job.multifile_segment;
     workarounds = job.workarounds;
-    max_segment_size = static_cast<off_t>(job.max_segment_size)
-                       * static_cast<off_t>(1024 * 1024);
+    run_in_frames = job.run_in_frames;
+    max_segment_size = static_cast<uint64_t>(job.max_segment_size)
+                       * static_cast<uint64_t>(1024 * 1024);
     max_PTS = static_cast<clockticks>(job.max_PTS) * CLOCKS;
 	video_delay = static_cast<clockticks>(job.video_offset);
 	audio_delay = static_cast<clockticks>(job.audio_offset);
@@ -117,6 +128,8 @@ void Multiplexor::InitSyntaxParameters(MultiplexJob &job)
             mjpeg_info( "Audio-only VCD track - variable-bit-rate (VCD2.0)");
             vbr = true;
         }
+        else
+            vbr = false;
 		break;
 		
 	case  MPEG_FORMAT_MPEG2 : 
@@ -134,9 +147,9 @@ void Multiplexor::InitSyntaxParameters(MultiplexJob &job)
 		always_buffers_in_audio = 1;
 		vcd_zero_stuffing = 0;
 		vbr = true;
-        	dtspts_for_all_vau = 0;
-        	timestamp_iframe_only = false;
-        	video_buffers_iframe_only = false;
+        dtspts_for_all_vau = 0;
+        timestamp_iframe_only = false;
+        video_buffers_iframe_only = false;
 		break;
 
 	case MPEG_FORMAT_SVCD :
@@ -270,8 +283,14 @@ void Multiplexor::InitSyntaxParameters(MultiplexJob &job)
 		sector_align_iframeAUs = false;
         timestamp_iframe_only = false;
         video_buffers_iframe_only = false;
+        vbr = false;
 		break;
 	}
+ 
+ if( job.VBR )
+     vbr = true;
+ if( job.CBR )
+     vbr = false;
 }
 
 /**************************************
@@ -543,7 +562,7 @@ segment_state;
  * ever process one frame at a time.
  * @returns the number of run-in sectors needed to fill up the buffers to suit the type of stream being muxed.
  */
-
+#if 0
 unsigned int Multiplexor::RunInSectors()
 {
 	std::vector<ElementaryStream *>::iterator str;
@@ -564,6 +583,64 @@ unsigned int Multiplexor::RunInSectors()
     sectors_delay += astreams.size();
 	return sectors_delay;
 }
+#endif
+clockticks Multiplexor::RunInDelay()
+{
+    std::vector<ElementaryStream *>::iterator str;
+    double frame_interval = 0.0;
+    clockticks delay;
+    
+    // User has specified a particular run-in
+
+    if(vstreams.size() != 0 )
+    {
+        frame_interval = CLOCKS / dynamic_cast<VideoStream *>(vstreams[0])->FrameRate();
+    }
+ 
+    if( run_in_frames != 0 ) 
+    {
+        if( frame_interval == 0.0 )
+        {
+            mjpeg_warn( "Run-in specified in frame intervals but no video stream - using 25Hz" );
+            frame_interval = CLOCKS / 25.0;
+        }
+        delay = static_cast<clockticks>(run_in_frames * frame_interval);
+    }
+    else
+    {
+        // No run-in specified: choose something reasonable based on the 
+        // specified buffer sizes
+        unsigned int data_delay = 0;
+
+        for( str = vstreams.begin(); str < vstreams.end(); ++str )
+        {
+            if( MPEG_STILLS_FORMAT( mux_format ) )
+            {
+                data_delay += static_cast<unsigned int>(1.1*(*str)->BufferSize());
+            }
+            else if( vbr )
+                data_delay += (*str)->BufferSize() / 2 ;
+            else
+                data_delay += 2*(*str)->BufferSize() / 3 ;
+        }
+        for( str = astreams.begin(); str < astreams.end(); ++str )
+        {
+            data_delay += 3*(*str)->BufferSize()/4;
+        }
+        ByteposTimecode( data_delay, delay );
+    }
+    
+    // Round delay a multiple of frame interval if its known...
+    if( frame_interval != 0.0 )
+    {
+        return static_cast<clockticks>(static_cast<int>( delay / frame_interval+0.5) * frame_interval);
+    }
+    else
+    {
+        return delay;
+    }
+
+}
 
 /**********************************************************************
  *
@@ -579,7 +656,6 @@ void Multiplexor::Init()
 {
 	std::vector<ElementaryStream *>::iterator str;
 	clockticks delay;
-	unsigned int sectors_delay;
 
 	Pack_struc 			dummy_pack;
 	Sys_header_struc 	dummy_sys_header;	
@@ -588,6 +664,8 @@ void Multiplexor::Init()
 	
 	mjpeg_info("SYSTEMS/PROGRAM stream:");
 	psstrm->Open();
+    if( vdr_index != 0 )
+        vdr_index->Open();
 	
     /* These are used to make (conservative) decisions
 	   about whether a packet should fit into the recieve buffers... 
@@ -700,11 +778,12 @@ void Multiplexor::Init()
 	   time to fill before decoding starts. Calculate the necessary delays...
 	*/
 
-	sectors_delay = RunInSectors();
-
-	ByteposTimecode( 
-		static_cast<bitcount_t>(sectors_delay*sector_transport_size),
-		delay );
+	//sectors_delay = RunInSectors();
+    //ByteposTimecode( 
+    //        static_cast<bitcount_t>(sectors_delay*sector_transport_size),
+    //        delay );
+    delay = RunInDelay();
+	
     video_delay += delay;
     audio_delay += delay;
 
@@ -720,8 +799,8 @@ void Multiplexor::Init()
         audio_delay += vstreams[0]->BasePTS()-vstreams[0]->BaseDTS();
     }
 
-	mjpeg_info( "Run-in Sectors = %d Video delay = %lld Audio delay = %lld",
-				sectors_delay,
+	mjpeg_info( "Run-in delay = %lld Video delay = %lld Audio delay = %lld",
+             delay / 300,
 				 video_delay / 300,
 				 audio_delay / 300 );
 
@@ -743,30 +822,39 @@ void Multiplexor::MuxStatus(log_level_t level)
 		switch( (*str)->Kind()  )
 		{
 		case ElementaryStream::video :
-			mjpeg_log( level,
-					   "Video %02x: buf=%7d frame=%06d sector=%08d",
-					   (*str)->stream_id,
-					   (*str)->BufferSize()-(*str)->bufmodel.Space(),
-					   (*str)->DecodeOrder(),
-					   (*str)->nsec
-				);
+            if( (*str)->MuxCompleted() )
+                mjpeg_log( level, "Video %02x: completed", (*str)->stream_id );
+            else
+			    mjpeg_log( level,
+					    "Video %02x: buf=%7d frame=%06d sector=%08d",
+					    (*str)->stream_id,
+					    (*str)->BufferSize()-(*str)->bufmodel.Space(),
+					    (*str)->DecodeOrder(),
+					    (*str)->nsec
+				    );
 			break;
 		case ElementaryStream::audio :
-			mjpeg_log( level,
-					   "Audio %02x: buf=%7d frame=%06d sector=%08d",
-					   (*str)->stream_id,
-					   (*str)->BufferSize()-(*str)->bufmodel.Space(),
-					   (*str)->DecodeOrder(),
-					   (*str)->nsec
-				);
+            if( (*str)->MuxCompleted() )
+                 mjpeg_log( level, "Audio %02x: completed", (*str)->stream_id );
+            else
+			    mjpeg_log( level,
+					    "Audio %02x: buf=%7d frame=%06d sector=%08d",
+					    (*str)->stream_id,
+					    (*str)->BufferSize()-(*str)->bufmodel.Space(),
+					    (*str)->DecodeOrder(),
+					    (*str)->nsec
+				    );
 			break;
 		default :
-			mjpeg_log( level,
-					   "Other %02x: buf=%7d sector=%08d",
-					   (*str)->stream_id,
-					   (*str)->bufmodel.Space(),
-					   (*str)->nsec
-				);
+            if( (*str)->MuxCompleted() )
+                 mjpeg_log( level, "Other %02x: completed", (*str)->stream_id );
+            else
+			    mjpeg_log( level,
+					    "Other %02x: buf=%7d sector=%08d",
+					    (*str)->stream_id,
+					    (*str)->bufmodel.Space(),
+					    (*str)->nsec
+				    );
 			break;
 		}
 	}
@@ -1064,7 +1152,7 @@ void Multiplexor::Multiplex()
 		case start_segment :
 			mjpeg_info( "New sequence commences..." );
 			SetPosAndSCR(0);
-			MuxStatus( LOG_INFO );
+			MuxStatus( mjpeg_loglev_t("info") );
 
 			for( str = estreams.begin(); str < estreams.end(); ++str )
 			{
@@ -1139,7 +1227,7 @@ void Multiplexor::Multiplex()
                                nextIframe->dorder,
                                runout_PTS/300, 
                                current_SCR/300 );
-                    MuxStatus( LOG_INFO );
+                    MuxStatus( mjpeg_loglev_t("info") );
 					running_out = true;
 					seg_state = runout_segment;
 				}
@@ -1170,15 +1258,18 @@ void Multiplexor::Multiplex()
 		for( str = estreams.begin(); str < estreams.end(); ++str )
 		{
 #ifdef STREAM_LOGGING
-            mjpeg_debug("%02x: SCR=%lld (%.3f) mux=%d %d reqDTS=%lld ", 
-                        (*str)->stream_id,
-                        current_SCR,
-                        static_cast<double>(current_SCR) /(90.0*300.0),
-                        (*str)->MuxPossible(current_SCR),
-                        (*str)->BufferSize()-(*str)->bufmodel.Space(),
-                        (*str)->RequiredDTS()/300
-                        
-				);
+            if( (*str)->MuxCompleted() )
+                mjpeg_debug( "%02x: complete", (*str)->stream_id );
+            else
+                mjpeg_debug("%02x: SCR=%lld (%.3f) mux=%d %d reqDTS=%lld ",
+                            (*str)->stream_id,
+                            current_SCR,
+                            static_cast<double>(current_SCR) /(90.0*300.0),
+                            (*str)->MuxPossible(current_SCR),
+                            (*str)->BufferSize()-(*str)->bufmodel.Space(),
+                           (*str)->RequiredDTS()/300
+                            
+				    );
 #endif
 			if( (*str)->MuxPossible(current_SCR) && 
 				( !video_first || (*str)->Kind() == ElementaryStream::video )
@@ -1205,7 +1296,7 @@ void Multiplexor::Multiplex()
 							despatch->stream_id, 
 							current_SCR/300, 
 							earliest/300 );
-				MuxStatus( LOG_WARN );
+				MuxStatus( mjpeg_loglev_t("warn") );
 				// Give the stream a chance to recover
 				underrun_ignore = 300;
 				++underruns;
@@ -1271,7 +1362,7 @@ void Multiplexor::Multiplex()
 				packets_left_in_pack = packets_per_pack;
 		}
 
-		MuxStatus( LOG_DEBUG );
+		MuxStatus( mjpeg_loglev_t("debug") );
 		/* Unless sys headers are always required we turn them off after the first
 		   packet has been generated */
 		include_sys_header = always_sys_header_in_pack;
@@ -1282,8 +1373,8 @@ void Multiplexor::Multiplex()
 		{
 			if( !(*pcomp) && (*str)->MuxCompleted() )
 			{
-				mjpeg_info( "STREAM %02x completed @ frame %d.", (*str)->stream_id, (*str)->DecodeOrder() );
-				MuxStatus( LOG_DEBUG );
+				mjpeg_info( "STREAM %02x completed", (*str)->stream_id );
+				MuxStatus( mjpeg_loglev_t("debug") );
 				(*pcomp) = true;
 			}
 			++str;
@@ -1294,8 +1385,11 @@ void Multiplexor::Multiplex()
 	
 	OutputSuffix( );
 	psstrm->Close();
+    if( vdr_index != 0)
+        vdr_index->Close();
+        
 	mjpeg_info( "Multiplex completion at SCR=%lld.", current_SCR/300);
-	MuxStatus( LOG_INFO );
+	MuxStatus( mjpeg_loglev_t("info") );
 	for( str = estreams.begin(); str < estreams.end(); ++str )
 	{
 		(*str)->Close();
@@ -1347,14 +1441,16 @@ unsigned int Multiplexor::PacketPayload( MuxStream &strm, bool buffers,
 @param returns the written bytes/packets (?)
 ***************************************************/
 
+struct VDRtIndex { uint32_t offset; uint8_t type; uint8_t number; uint16_t reserved; };
+
 unsigned int 
 Multiplexor::WritePacket( unsigned int     max_packet_data_size,
-                           MuxStream        &strm,
-                           bool 	 buffers,
-                           clockticks   	 PTS,
-                           clockticks   	 DTS,
-                           uint8_t 	 timestamps
-	)
+                                    MuxStream        &strm,
+                                    bool 	 buffers,
+                                    clockticks   	 PTS,
+                                    clockticks   	 DTS,
+                                    uint8_t 	 timestamps
+	                     )
 {
     unsigned int written =
         psstrm->CreateSector ( pack_header_ptr,
@@ -1370,6 +1466,42 @@ Multiplexor::WritePacket( unsigned int     max_packet_data_size,
     return written;
 }
 
+/***************************************************
+
+  IndexLastPacket 
+  - Generate the index data (if any) for the latest
+  packet generated in the specified stream.
+  N.b. should usually called immediately after WritePacket.
+  Its not part of WritePacket because it is not needed
+  for all stream types...
+***************************************************/
+void
+Multiplexor::IndexLastPacket( ElementaryStream &strm, int index_type )
+{
+    switch( strm.Kind() )
+    {
+        case ElementaryStream::video :
+            if( index_type != NOFRAME && vdr_index != 0 )
+            {
+                union _ibuf 
+                {
+                        uint8_t bytes[sizeof(VDRtIndex)];
+                        VDRtIndex istruct;     
+                } indexbuf;
+                
+                indexbuf.istruct.offset = (int)psstrm->LastPackStart();
+                indexbuf.istruct.type = index_type;
+                indexbuf.istruct.number = (int)psstrm->SegmentNum();
+                indexbuf.istruct.reserved = 0;
+                vdr_index->Write( indexbuf.bytes, sizeof(VDRtIndex) );
+                
+            }
+	break;
+        default :
+            abort(); // Currently only video indexing implemented
+    }
+}
+
 /***************************************************
  *
  * WriteRawSector - Write out a packet carrying data for
@@ -1435,7 +1567,9 @@ void Multiplexor::OutputPadding (bool vcd_audio_pad)
  *	OutputGOPControlSector
  *  DVD System headers are carried in peculiar sectors carrying 2
  *  PrivateStream2 packets.   We're sticking 0's in the packets
- *  as we have no idea what's supposed to be in there.
+ * for anything other than the substream IDs as they're
+ * merely being inserted as place-holders to provide gaps the
+ * DVD authoring SW can fill in later.
  *
  * Thanks to Brent Byeler who worked out this work-around.
  *
@@ -1448,11 +1582,11 @@ void Multiplexor::OutputDVDPriv2 (	)
     uint8_t *sector_buf = new uint8_t[sector_size];
     unsigned int tozero;
     assert( sector_size == 2048 );
-    PS_Stream::BufferSectorHeader( sector_buf,
+    psstrm->BufferSectorHeader( sector_buf,
                                 pack_header_ptr,
                                 &sys_header,
                                 index );
-    PS_Stream::BufferPacketHeader( index,
+    psstrm->BufferPacketHeader( index,
                                    PRIVATE_STR_2,
                                    2,      // MPEG 2
                                    false,  // No buffers
@@ -1466,10 +1600,11 @@ void Multiplexor::OutputDVDPriv2 (	)
                                    index );
     tozero = sector_buf+1024-index;
     memset( index, 0, tozero);
+    index[0] = 0x00; // Substream 1 (PCI)
     index += tozero;
-    PS_Stream::BufferPacketSize( packet_size_field, index );    
+    psstrm->BufferPacketSize( packet_size_field, index );    
 
-    PS_Stream::BufferPacketHeader( index,
+    psstrm->BufferPacketHeader( index,
                                    PRIVATE_STR_2,
                                    2,      // MPEG 2
                                    false,  // No buffers
@@ -1483,8 +1618,9 @@ void Multiplexor::OutputDVDPriv2 (	)
                                    index );
     tozero = sector_buf+2048-index;
     memset( index, 0, tozero );
+    index[0] = 0x01; // Substream 1 (DSI)
     index += tozero;
-    PS_Stream::BufferPacketSize( packet_size_field, index );
+    psstrm->BufferPacketSize( packet_size_field, index );
 
     WriteRawSector( sector_buf, sector_size );
 
