@@ -1,5 +1,5 @@
 /* 
- * $Id: y4munsharp.c,v 1.6 2005/04/11 18:16:42 sms00 Exp $
+ * $Id: y4munsharp.c,v 1.10 2007/04/01 17:21:57 sms00 Exp $
  *
  * Constructed using:
  * unsharp.c 0.10 -- This is a plug-in for the GIMP 1.0
@@ -35,11 +35,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#ifdef	HAVE_CONFIG_H
 #include "config.h"
-#else
-#define HAVE_STDINT_H
-#endif
 #include <stdlib.h>
 #include <math.h>
 #include <unistd.h>
@@ -62,7 +58,7 @@ static int gen_convolve_matrix(double, double **);
 
 	u_char	*i_yuv[3], *o_yuv[3], *cur_col, *dest_col, *cur_row, *dest_row;
 
-	double	y_radius = 3.0, y_amount = 0.30;
+	double	y_radius = 2.0, y_amount = 0.30;
 	double	uv_radius = -1.0, uv_amount;
 	int	y_threshold = 4, uv_threshold, frameno;
 
@@ -186,14 +182,14 @@ main(int argc, char **argv)
 	y4m_copy_stream_info(&ostream, &istream);
 	y4m_write_stream_header(fileno(stdout), &ostream);
 
-	mjpeg_log(LOG_INFO, "Luma radius: %lf", y_radius);
-	mjpeg_log(LOG_INFO, "Luma amount: %lf", y_amount);
-	mjpeg_log(LOG_INFO, "Luma threshold: %d", y_threshold);
+	mjpeg_info("Luma radius: %f", y_radius);
+	mjpeg_info("Luma amount: %f", y_amount);
+	mjpeg_info("Luma threshold: %d", y_threshold);
 	if	(uv_radius != -1.0)
 		{
-		mjpeg_log(LOG_INFO, "Chroma radius: %lf", uv_radius);
-		mjpeg_log(LOG_INFO, "Chroma amount: %lf", uv_amount);
-		mjpeg_log(LOG_INFO, "Chroma threshold: %d", uv_threshold);
+		mjpeg_info("Chroma radius: %f", uv_radius);
+		mjpeg_info("Chroma amount: %f", uv_amount);
+		mjpeg_info("Chroma threshold: %d", uv_threshold);
 		}
 
 	for	(frameno = 0; y4m_read_frame(fdin, &istream, &iframe, i_yuv) == Y4M_OK; frameno++)
@@ -223,7 +219,7 @@ void y4munsharp(void)
 	int	i, row, col, diff, value;
 	u_char	*i_ptr, *o_ptr;
 
-	mjpeg_log(LOG_DEBUG, "Blurring Luma rows frame %d", frameno);
+	mjpeg_debug("Blurring Luma rows frame %d", frameno);
 
 	for	(row = 0; row < yheight; row++)
 		{
@@ -235,7 +231,7 @@ void y4munsharp(void)
 
 	if	(uv_radius != -1.0)
 		{
-		mjpeg_log(LOG_DEBUG, "Blurring Chroma rows frame %d", frameno);
+		mjpeg_debug("Blurring Chroma rows frame %d", frameno);
 		for	(row = 0; row < uvheight; row++)
 			{
 			blur_line(ctable_uv, cmatrix_uv, cmatrix_uv_len,
@@ -254,7 +250,7 @@ void y4munsharp(void)
 		memcpy(o_yuv[2], i_yuv[2], uvlen);
 		}
 
-	mjpeg_log(LOG_DEBUG, "Blurring Luma columns frame %d", frameno);
+	mjpeg_debug("Blurring Luma columns frame %d", frameno);
 	for	(col = 0; col < ywidth; col++)
 		{
 /*
@@ -294,7 +290,7 @@ void y4munsharp(void)
 	if	(uv_radius == -1)
 		goto merging;
 
-	mjpeg_log(LOG_DEBUG, "Blurring chroma columns frame %d", frameno);
+	mjpeg_debug("Blurring chroma columns frame %d", frameno);
 	for	(col = 0; col < uvwidth; col++)
 		{
 /* U */
@@ -347,7 +343,7 @@ void y4munsharp(void)
 			}
 		}
 merging:
-	mjpeg_log(LOG_DEBUG, "Merging luma frame %d", frameno);
+	mjpeg_debug("Merging luma frame %d", frameno);
 	for	(row = 0, i_ptr = i_yuv[0], o_ptr = o_yuv[0]; row < yheight; row++)
 		{
 		for	(i = 0; i < ywidth; i++, i_ptr++, o_ptr++)
@@ -370,7 +366,7 @@ merging:
 	if	(uv_radius == -1.0)
 		goto done;
 
-	mjpeg_log(LOG_DEBUG, "Merging chroma frame %d", frameno);
+	mjpeg_debug("Merging chroma frame %d", frameno);
 	for	(row = 0, i_ptr = i_yuv[1], o_ptr = o_yuv[1]; row < uvheight; row++)
 		{
 		for	(i = 0; i < uvwidth; i++, i_ptr++, o_ptr++)
@@ -640,7 +636,7 @@ void usage(void)
 	fprintf(stderr, "%s:\tradius and amount are floating point numbers\n",
 		__progname);
 	fprintf(stderr, "%s:\tthreshold is integer.\n", __progname);
-	fprintf(stderr, "%s:\tdefault for -L is 3.0,0.3,4\n", __progname);
+	fprintf(stderr, "%s:\tdefault for -L is 2.0,0.3,4\n", __progname);
 	fprintf(stderr, "%s:\tchroma not filtered UNLESS -C used, no default\n",
 		__progname);
 	fprintf(stderr, "%s:-v verbose 0=quiet 1=normal 2=debug (default: 1)\n", __progname);
